@@ -35,6 +35,7 @@
 #include <ctype.h>
 #include <errno.h>
 #include <netinet/in.h>
+#include <signal.h>
 #include <stdarg.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -56,6 +57,11 @@
 #include "nfxV3.h"
 #include "util.h"
 
+typedef struct finaliseArgs_s {
+    nffile_t *nffile;
+    uint32_t badPacket;
+} finaliseArgs_t;
+
 /* local variables */
 static uint32_t exporter_sysid = 0;
 static char *DynamicSourcesDir = NULL;
@@ -193,13 +199,15 @@ int AddFlowSource(FlowSource_t **FlowSource, char *ident, char *ip, char *flowpa
     (*source)->datadir = path;
 
     char s[MAXPATHLEN];
-    // cache current collector file
-    if (snprintf(s, MAXPATHLEN - 1, "%s/%s.%lu", (*source)->datadir, NF_DUMPFILE, (unsigned long)getpid()) >= (MAXPATHLEN - 1)) {
-        LogError("Path too long: %s", flowpath);
+    // cache current generic collector file
+    if (snprintf(s, MAXPATHLEN - 1, "%s/%s.XXXXXX", (*source)->datadir, NF_TMPFILE) >= (MAXPATHLEN - 1)) {
+        LogError("Path too long: %s", (*source)->datadir);
         return 0;
     }
-    (*source)->current = strdup(s);
-    if (!(*source)->current) {
+
+    // generic tmp filename
+    (*source)->tmpFileName = strdup(s);
+    if (!(*source)->tmpFileName) {
         LogError("strdup() error: %s", strerror(errno));
         return 0;
     }
@@ -235,7 +243,7 @@ int AddFlowSourceString(FlowSource_t **FlowSource, char *argument) {
 FlowSource_t *AddDynamicSource(FlowSource_t **FlowSource, struct sockaddr_storage *ss) {
     FlowSource_t **source;
     void *ptr;
-    char *s, ident[100], path[MAXPATHLEN];
+    char *s, ident[100];
     int err;
 
     union {
@@ -331,6 +339,7 @@ FlowSource_t *AddDynamicSource(FlowSource_t **FlowSource, struct sockaddr_storag
     strncpy((*source)->Ident, ident, IDENTLEN - 1);
     (*source)->Ident[IDENTLEN - 1] = '\0';
 
+    char path[MAXPATHLEN];
     snprintf(path, MAXPATHLEN - 1, "%s/%s", DynamicSourcesDir, ident);
     path[MAXPATHLEN - 1] = '\0';
 
@@ -343,15 +352,16 @@ FlowSource_t *AddDynamicSource(FlowSource_t **FlowSource, struct sockaddr_storag
     }
     (*source)->datadir = strdup(path);
 
-    if (snprintf(path, MAXPATHLEN - 1, "%s/%s.%lu", (*source)->datadir, NF_DUMPFILE, (unsigned long)getpid()) >= (MAXPATHLEN - 1)) {
+    // cache current generic collector file
+    if (snprintf(path, MAXPATHLEN - 1, "%s/%s.XXXXXX", (*source)->datadir, NF_TMPFILE) >= (MAXPATHLEN - 1)) {
         LogError("Path too long: %s\n", path);
         free(*source);
         *source = NULL;
         return NULL;
     }
-    (*source)->current = strdup(path);
+    (*source)->tmpFileName = strdup(path);
 
-    LogInfo("Dynamically add source ident: %s in directory: %s", ident, path);
+    LogInfo("Dynamically add source ident: %s in directory: %s", ident, (*source)->datadir);
     return *source;
 
 }  // End of AddDynamicSource
@@ -376,15 +386,13 @@ int RotateFlowFiles(time_t t_start, char *time_extension, FlowSource_t *fs, int
     // for each flow source update the stats, close the file and re-initialize the new file
     while (fs) {
         char nfcapd_filename[MAXPATHLEN];
-        char error[255];
         nffile_t *nffile = fs->nffile;
 
         // prepare filename
         if (subdir) {
-            if (SetupSubDir(fs->datadir, subdir, error, 255)) {
+            if (SetupSubDir(fs->datadir, subdir)) {
                 snprintf(nfcapd_filename, MAXPATHLEN - 1, "%s/%s/nfcapd.%s", fs->datadir, subdir, fmt);
             } else {
-                LogError("Ident: %s, Failed to create sub hier directories: %s", fs->Ident, error);
                 // skip subdir - put flows directly into current directory
                 snprintf(nfcapd_filename, MAXPATHLEN - 1, "%s/nfcapd.%s", fs->datadir, fmt);
             }
@@ -396,23 +404,28 @@ int RotateFlowFiles(time_t t_start, char *time_extension, FlowSource_t *fs, int
         // update stat record
         // if no flows were collected, fs->msecLast is still 0
         // set msecFirst and msecLast and to start of this time slot
-        if (fs->msecLast == 0) {
-            fs->msecFirst = 1000LL * (uint64_t)t_start;
-            fs->msecLast = fs->msecFirst;
+        if (fs->nffile->stat_record->msecLastSeen == 0) {
+            fs->nffile->stat_record->msecFirstSeen = 1000LL * (uint64_t)t_start;
+            fs->nffile->stat_record->msecLastSeen = fs->nffile->stat_record->msecFirstSeen;
         }
-        nffile->stat_record->firstseen = fs->msecFirst;
-        nffile->stat_record->lastseen = fs->msecLast;
 
         // Flush Exporter Stat to file
         FlushExporterStats(fs);
         // Flush open datablock
         fs->dataBlock = WriteBlock(fs->nffile, fs->dataBlock);
+
+        // need tmp filename for renaming - Closing the file, discards the filename
+        char tmpFilename[MAXPATHLEN];
+        strncpy(tmpFilename, fs->nffile->fileName, MAXPATHLEN - 1);
+        tmpFilename[MAXPATHLEN - 1] = '\0';
+
         // Close file
-        CloseUpdateFile(nffile);
+        FinaliseFile(nffile);
+        CloseFile(nffile);
 
         // if rename fails, we are in big trouble, as we need to get rid of the old .current
         // file otherwise, we will loose flows and can not continue collecting new flows
-        if (RenameAppend(fs->current, nfcapd_filename) < 0) {
+        if (RenameAppend(tmpFilename, nfcapd_filename) < 0) {
             LogError("Ident: %s, Can't rename dump file: %s", fs->Ident, strerror(errno));
 
             // we do not update the books here, as the file failed to rename properly
@@ -432,11 +445,9 @@ int RotateFlowFiles(time_t t_start, char *time_extension, FlowSource_t *fs, int
 
         // reset stats
         fs->bad_packets = 0;
-        fs->msecFirst = 0xffffffffffffLL;
-        fs->msecLast = 0;
 
         if (!done) {
-            fs->nffile = OpenNewFile(fs->current, fs->nffile, CREATOR_NFCAPD, INHERIT, INHERIT);
+            fs->nffile = OpenNewFile(SetUniqueTmpName(fs->tmpFileName), fs->nffile, CREATOR_NFCAPD, INHERIT, INHERIT);
             if (!fs->nffile) {
                 LogError("killed due to fatal error: ident: %s", fs->Ident);
                 return 0;
