@@ -80,12 +80,22 @@ typedef struct ip_addr_s {
 typedef struct exporter_info_record_s exporter_info_record_t;
 typedef struct extension_map_s extension_map_t;
 
-enum { EXlocal = MAXEXTENSIONS, EXheader, SSLindex, JA3index, JA4index, MAXLISTSIZE };
+enum { EXlocal = MAXEXTENSIONS, EXheader, EXinPayloadHandle, EXoutPayloadHandle, MAXLISTSIZE };
+
+typedef struct payloadHandle_s {
+    // use opaque types
+    void *dns;
+    void *ssl;
+    char *ja3;
+    void *ja4;
+} payloadHandle_t;
 
 typedef struct recordHandle_s {
     recordHeaderV3_t *recordHeaderV3;
     void *extensionList[MAXLISTSIZE];
     char geo[16];
+#define OFFinPayload offsetof(recordHandle_t, inPayload)
+#define OFFoutPayload offsetof(recordHandle_t, outPayload)
 #define OFFgeo offsetof(recordHandle_t, geo)
 #define OFFgeoSrcIP offsetof(recordHandle_t, geo)
 #define OFFgeoDstIP offsetof(recordHandle_t, geo) + 2
@@ -123,8 +133,8 @@ typedef struct stat_record_s {
     uint64_t numpackets_icmp;
     uint64_t numpackets_other;
     // time window
-    uint64_t firstseen;
-    uint64_t lastseen;
+    uint64_t msecFirstSeen;
+    uint64_t msecLastSeen;
     // other
     uint64_t sequence_failure;
 } stat_record_t;
