@@ -36,9 +36,22 @@ static inline int MapRecordHandle(recordHandle_t *handle, recordHeaderV3_t *reco
 static inline dataBlock_t *AppendToBuffer(nffile_t *nffile, dataBlock_t *dataBlock, void *record, size_t required);
 
 static inline int MapRecordHandle(recordHandle_t *handle, recordHeaderV3_t *recordHeaderV3, uint64_t flowCount) {
-    if (handle->extensionList[SSLindex]) free(handle->extensionList[SSLindex]);
-    if (handle->extensionList[JA3index]) free(handle->extensionList[JA3index]);
-    if (handle->extensionList[JA4index]) free(handle->extensionList[JA4index]);
+    payloadHandle_t *payloadHandle = (payloadHandle_t *)handle->extensionList[EXinPayloadHandle];
+    if (payloadHandle) {
+        if (payloadHandle->dns) free(payloadHandle->dns);
+        if (payloadHandle->ssl) free(payloadHandle->ssl);
+        if (payloadHandle->ja3) free(payloadHandle->ja3);
+        if (payloadHandle->ja4) free(payloadHandle->ja4);
+        free(payloadHandle);
+    }
+    payloadHandle = (payloadHandle_t *)handle->extensionList[EXoutPayloadHandle];
+    if (payloadHandle) {
+        if (payloadHandle->dns) free(payloadHandle->dns);
+        if (payloadHandle->ssl) free(payloadHandle->ssl);
+        if (payloadHandle->ja3) free(payloadHandle->ja3);
+        if (payloadHandle->ja4) free(payloadHandle->ja4);
+        free(payloadHandle);
+    }
 
     memset((void *)handle, 0, sizeof(recordHandle_t));
     handle->recordHeaderV3 = recordHeaderV3;
@@ -81,6 +94,28 @@ static inline int MapRecordHandle(recordHandle_t *handle, recordHeaderV3_t *reco
             if (natCommon) genericFlow->msecFirst = natCommon->msecEvent;
         }
     }
+    EXlayer2_t *EXlayer2 = (EXlayer2_t *)handle->extensionList[EXlayer2ID];
+    if (EXlayer2) {
+        // Honor the IPversion flag and mask out the unneeded extension
+        switch (EXlayer2->ipVersion) {
+            case 0:  // not present - skip
+                break;
+            case 4:
+                if (handle->extensionList[EXipv6FlowID]) {
+                    handle->extensionList[EXipv6FlowID] = NULL;
+                    recordHeaderV3->numElements--;
+                }
+                break;
+            case 6:
+                if (handle->extensionList[EXipv4FlowID]) {
+                    handle->extensionList[EXipv4FlowID] = NULL;
+                    recordHeaderV3->numElements--;
+                }
+                break;
+            default:
+                LogError("Mapping record: %" PRIu64 "  - Error - unknown IP version: %d", flowCount, EXlayer2->ipVersion);
+        }
+    }
     return 1;
 }
 
