@@ -83,6 +83,8 @@ typedef enum {
     CMP_PAYLOAD,
     CMP_REGEX,
     CMP_GEO,
+    CMP_DNSNAME,
+    CMP_DNSIP,
 } comparator_t;
 
 typedef struct FilterParam {
@@ -107,7 +109,7 @@ typedef enum {
     FUNC_BPP,          // function code for bpp ( bytes per packet ) filter function
     FUNC_MPLS_LABEL,   // function code for matching an MPLS label
     FUNC_MPLS_EOS,     // function code for matching End of MPLS Stack label
-    FUNC_MPLS_EXP,     // function code for matching experimantal value in label
+    FUNC_MPLS_EXP,     // function code for matching experimental value in label
     FUNC_MPLS_ANY,     // function code for matching any MPLS label
     FUNC_PBLOCK,       // function code for matching ports against pblock start
     FUNC_MMAS_LOOKUP,  // function code for optional maxmind AS lookup
@@ -116,6 +118,14 @@ typedef enum {
     FUNC_TTL_EQUAL,    // function code for comparing min/max TTL
 } filterFunction_t;
 
+typedef enum {
+    OPT_NONE = 0,  // no option
+    OPT_DNS,       // payload processing for DNS
+    OPT_SSL,       // payload processing for SSL
+    OPT_JA3,       // payload processing for ja3
+    OPT_JA4,       // payload processing for ja4
+} filterOption_t;
+
 #define FULLMASK FFFFFFFFFFFFFFFFLL
 
 /* Definition of the IP list node */
@@ -153,6 +163,8 @@ int yylex(void);
 
 uint32_t NewElement(uint32_t extID, uint32_t offset, uint32_t length, uint64_t value, comparator_t comp, filterFunction_t function, data_t data);
 
+void SetElementOption(uint32_t elementID, filterOption_t option);
+
 uint32_t Invert(uint32_t a);
 
 uint32_t Connect_OR(uint32_t b1, uint32_t b2);
