@@ -184,11 +184,11 @@
  * %F    is equivalent to ``%Y-%m-%d''.
  */
 
-// predefined and accpeted formats
+// predefined and accepted formats
 static const char *subdir_def[] = {"",  // default index 0 - no subdir hierarchy
                                    "%Y/%m/%d", "%Y/%m/%d/%H", "%Y/%W/%u", "%Y/%W/%u/%H", "%Y/%j", "%Y/%j/%H", "%F", "%F/%H", NULL};
 
-// all accpeted char in a string
+// all accepted char in a string
 #define AcceptedFormatChar "YymdjHMsUWwuF"
 
 static mode_t mode, dir_mode;
@@ -774,7 +774,7 @@ static int GetFileList(char *path, timeWindow_t *timeWindow) {
     // last entry must be NULL
     InsertString(&source_dirs, NULL);
     if (!source_dirs.list) {
-        LogError("ERROR: No sourc dir at %s line %d", __FILE__, __LINE__);
+        LogError("ERROR: No source dir at %s line %d", __FILE__, __LINE__);
         return 0;
     }
     fts = fts_open(source_dirs.list, FTS_LOGICAL, compare);
@@ -814,7 +814,7 @@ static int GetFileList(char *path, timeWindow_t *timeWindow) {
                 // file entry
 
                 // skip stat file
-                if (strcmp(ftsent->fts_name, ".nfstat") == 0 || strncmp(ftsent->fts_name, NF_DUMPFILE, strlen(NF_DUMPFILE)) == 0) continue;
+                if (strcmp(ftsent->fts_name, ".nfstat") == 0 || strncmp(ftsent->fts_name, NF_TMPFILE, strlen(NF_TMPFILE)) == 0) continue;
                 if (strstr(ftsent->fts_name, ".stat") != NULL) continue;
                 // skip OSX DS_Store files
                 if (strstr(ftsent->fts_name, ".DS_Store") != NULL) continue;
@@ -1108,7 +1108,7 @@ static char *GuessSubDir(char *channeldir, char *filename) {
 }  // End of GuessSubDir
 
 char *GetSubDir(struct tm *now) {
-    static char subpath[255];
+    static __thread char subpath[255];
     size_t sublen;
 
     sublen = strftime(subpath, 254, subdir_format, now);
@@ -1119,14 +1119,12 @@ char *GetSubDir(struct tm *now) {
 
 int GetSubDirIndex(void) { return subdir_index; }
 
-int SetupSubDir(char *dir, char *subdir, char *error, size_t errlen) {
+int SetupSubDir(char *dir, char *subdir) {
     char *p, path[MAXPATHLEN];
     struct stat stat_buf;
     size_t sublen, pathlen;
     int err;
 
-    error[0] = '\0';
-
     path[0] = '\0';
     strncat(path, dir, MAXPATHLEN - 1);
     path[MAXPATHLEN - 1] = '\0';
@@ -1135,7 +1133,7 @@ int SetupSubDir(char *dir, char *subdir, char *error, size_t errlen) {
     pathlen = strlen(path);
     // set p as reference between path and subdir
     if ((sublen + pathlen + 2) >= (MAXPATHLEN - 1)) {  // +2 : add 1 for '/'
-        snprintf(error, errlen, "Path '%s': too long", path);
+        LogError("SetupSubDir(): path '%s': too long", path);
         return 0;
     }
 
@@ -1152,7 +1150,7 @@ int SetupSubDir(char *dir, char *subdir, char *error, size_t errlen) {
             return 1;
         } else {
             // an entry with this name exists, but it's not a directory
-            snprintf(error, errlen, "Path '%s': %s ", path, strerror(ENOTDIR));
+            LogError("SetupSubDir(): path '%s': %s ", path, strerror(ENOTDIR));
             return 0;
         }
     }
@@ -1164,11 +1162,15 @@ int SetupSubDir(char *dir, char *subdir, char *error, size_t errlen) {
 
     // else errno is set
     if (errno == ENOENT) {  // we need to create intermediate directories as well
-        err = mkpath(path, p, mode, dir_mode, error, errlen);
-        if (err == 0)  // creation was successful
+        char error[255];
+        err = mkpath(path, p, mode, dir_mode, error, 255);
+        if (err == 0) {  // creation was successful
             return 1;
+        } else {
+            LogError("mkpath() error for Path: '%s': %s", path, error);
+        }
     } else {
-        snprintf(error, errlen, "mkdir() error for '%s': %s", path, strerror(errno));
+        LogError("mkdir() error for '%s': %s", path, strerror(errno));
     }
 
     // anything else failed and error string is set
@@ -1199,11 +1201,11 @@ static int mkpath(char *path, char *p, mode_t mode, mode_t dir_mode, char *error
 
         if (stat(path, &sb)) {
             if (errno != ENOENT || (mkdir(path, done ? mode : dir_mode) && errno != EEXIST)) {
-                snprintf(error, errlen, "mkdir() error for '%s': %s", path, strerror(errno));
+                snprintf(error, errlen, "mkdir() error: %s", strerror(errno));
                 return (-1);
             }
         } else if (!S_ISDIR(sb.st_mode)) {
-            snprintf(error, errlen, "Path '%s': %s ", path, strerror(ENOTDIR));
+            snprintf(error, errlen, "%s", strerror(ENOTDIR));
             return (-1);
         }
 
@@ -1223,8 +1225,8 @@ static int CheckTimeWindow(char *filename, timeWindow_t *searchWindow) {
         return 0;
     }
 
-    if (searchWindow->msecLast && searchWindow->msecLast < stat_record.firstseen) return 0;
-    if (searchWindow->msecFirst && searchWindow->msecFirst > stat_record.lastseen) return 0;
+    if (searchWindow->msecLast && searchWindow->msecLast < stat_record.msecFirstSeen) return 0;
+    if (searchWindow->msecFirst && searchWindow->msecFirst > stat_record.msecLastSeen) return 0;
 
     return 1;
 
