@@ -98,6 +98,8 @@ static const char *nf_creator[MAX_CREATOR] = {"unknown", "nfcapd",    "nfpcapd",
 
 static unsigned NumWorkers = DEFAULTWORKERS;
 
+static uint32_t pseudoID = 0;
+
 /* function prototypes */
 static int LZO_initialize(void);
 
@@ -125,13 +127,19 @@ static int Uncompress_Block_BZ2(dataBlock_t *in_block, dataBlock_t *out_block, s
 
 static dataBlock_t *nfread(nffile_t *nffile);
 
+static void *nfwriter(void *arg);
+
+static void *nfreader(void *arg);
+
 static int nfwrite(nffile_t *nffile, dataBlock_t *block_header);
 
 static int ReadAppendix(nffile_t *nffile);
 
 static int WriteAppendix(nffile_t *nffile);
 
-static int SignalTerminate(nffile_t *nffile);
+static void joinWorkers(nffile_t *nffile);
+
+static void TerminateWorkers(nffile_t *nffile);
 
 static void FlushFile(nffile_t *nffile);
 
@@ -169,6 +177,9 @@ int Init_nffile(int workers, queue_t *fileList) {
     atomic_init(&blocksInUse, 0);
 
     NumWorkers = GetNumWorkers(workers);
+
+    pseudoID = time(NULL) ^ 0x557aa755;
+
     return 1;
 
 }  // End of Init_nffile
@@ -644,7 +655,7 @@ static int WriteAppendix(nffile_t *nffile) {
     recordHeader->type = TYPE_STAT;
     recordHeader->size = sizeof(recordHeader_t) + sizeof(stat_record_t);
     // in case of an empty stat record
-    if (nffile->stat_record->firstseen == 0x7fffffffffffffffLL) nffile->stat_record->firstseen = 0;
+    if (nffile->stat_record->msecFirstSeen == 0x7fffffffffffffffLL) nffile->stat_record->msecFirstSeen = 0;
     memcpy(data, nffile->stat_record, sizeof(stat_record_t));
 
     block_header->NumRecords++;
@@ -654,6 +665,9 @@ static int WriteAppendix(nffile_t *nffile) {
     nfwrite(nffile, block_header);
     FreeDataBlock(block_header);
 
+    // NumBlocks are plain data blocks - subtract appendix blocks
+    nffile->file_header->NumBlocks -= nffile->file_header->appendixBlocks;
+
     return 1;
 
 }  // End of WriteAppendix
@@ -714,10 +728,9 @@ nffile_t *NewFile(nffile_t *nffile) {
         nffile->ident = NULL;
     }
     memset((void *)nffile->stat_record, 0, sizeof(stat_record_t));
-    nffile->stat_record->firstseen = 0x7fffffffffffffff;
+    nffile->stat_record->msecFirstSeen = 0x7fffffffffffffff;
 
     for (int i = 0; i < MAXWORKERS; i++) nffile->worker[i] = 0;
-    atomic_store(&nffile->terminate, 0);
     pthread_mutex_init(&nffile->wlock, NULL);
     return nffile;
 
@@ -889,7 +902,6 @@ nffile_t *OpenFile(char *filename, nffile_t *nffile) {
     // kick off nfreader
     // there is only 1 reader thread -> slot 0
     pthread_t tid;
-    atomic_store(&nffile->terminate, 0);
     queue_open(nffile->processQueue);
     int err = pthread_create(&tid, NULL, nfreader, (void *)nffile);
     if (err) {
@@ -904,9 +916,9 @@ nffile_t *OpenFile(char *filename, nffile_t *nffile) {
 
 // Create a new nffile
 //  filename   : full path of file to create
-//  nffile     : Use nffile handle and initialize it accordingly. If NULL a new handle is alocated
-//  creater    : Creator ID
-//  compress   : Compression alforithm and level. Lower 16bit: algo. Upper 16bit level
+//  nffile     : Use nffile handle and initialize it accordingly. If NULL a new handle is allocated
+//  creator    : Creator ID
+//  compress   : Compression algorithm and level. Lower 16bit: algo. Upper 16bit level
 //  encryption : Encryption algorithm used.
 nffile_t *OpenNewFile(char *filename, nffile_t *nffile, int creator, int compress, int encryption) {
     int fd;
@@ -964,7 +976,6 @@ nffile_t *OpenNewFile(char *filename, nffile_t *nffile, int creator, int compres
     }
 
     // kick off nfwriter
-    atomic_store(&nffile->terminate, 0);
     queue_open(nffile->processQueue);
 
     // if file is not compressed, 2 workers are fine.
@@ -1023,7 +1034,6 @@ nffile_t *AppendFile(char *filename) {
     }
 
     // kick off NumWorkers nfwriter threads
-    atomic_store(&nffile->terminate, 0);
     queue_open(nffile->processQueue);
 
     unsigned NumThreads = nffile->file_header->compression == 0 ? 1 : NumWorkers;
@@ -1063,7 +1073,8 @@ int RenameAppend(char *oldName, char *newName) {
         SumStatRecords(nffile_w->stat_record, nffile_r->stat_record);
         DisposeFile(nffile_r);
 
-        CloseUpdateFile(nffile_w);
+        FinaliseFile(nffile_w);
+        CloseFile(nffile_w);
         DisposeFile(nffile_w);
 
         return unlink(oldName);
@@ -1085,30 +1096,24 @@ static void FlushFile(nffile_t *nffile) {
     queue_sync(nffile->processQueue);
     // writers terminate, on queue closed and empty
 
-    // wait for all nfwriter threads to exit
-    for (unsigned i = 0; i < NumWorkers; i++) {
-        if (nffile->worker[i]) {
-            int err = pthread_join(nffile->worker[i], NULL);
-            if (err) {
-                LogError("pthread_join() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
-            }
-            nffile->worker[i] = 0;
-        }
-    }
+    joinWorkers(nffile);
+
     fsync(nffile->fd);
 
 }  // End of FlushFile
 
+// wait for all queue entries have been processed and queuen is empty
+void SyncFile(nffile_t *nffile) {
+    if (!nffile || nffile->fd == 0) return;
+    queue_sync(nffile->processQueue);
+}  // End of SyncFile
+
 // close reading file
 void CloseFile(nffile_t *nffile) {
     if (!nffile || nffile->fd == 0) return;
 
     // make sure all workers are gone
-    for (unsigned i = 0; i < NumWorkers; i++) {
-        if (nffile->worker[i]) {
-            SignalTerminate(nffile);
-        }
-    }
+    TerminateWorkers(nffile);
 
     close(nffile->fd);
     nffile->fd = 0;
@@ -1124,7 +1129,6 @@ void CloseFile(nffile_t *nffile) {
     }
 
     // clean queue
-    queue_close(nffile->processQueue);
     while (queue_length(nffile->processQueue)) {
         dataBlock_t *block_header = queue_pop(nffile->processQueue);
         FreeDataBlock(block_header);
@@ -1134,7 +1138,7 @@ void CloseFile(nffile_t *nffile) {
 }  // End of CloseFile
 
 // close writing file
-int CloseUpdateFile(nffile_t *nffile) {
+int FinaliseFile(nffile_t *nffile) {
     FlushFile(nffile);
 
     if (!WriteAppendix(nffile)) {
@@ -1147,9 +1151,6 @@ int CloseUpdateFile(nffile_t *nffile) {
         return 0;
     }
 
-    // NumBlocks are plain data blocks - subtract appendix blocks
-    nffile->file_header->NumBlocks -= nffile->file_header->appendixBlocks;
-
     if (write(nffile->fd, (void *)nffile->file_header, sizeof(fileHeaderV2_t)) <= 0) {
         LogError("write() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return 0;
@@ -1160,12 +1161,10 @@ int CloseUpdateFile(nffile_t *nffile) {
         close(nffile->fd);
         return 0;
     }
-    fsync(nffile->fd);
-    CloseFile(nffile);
 
     return 1;
 
-} /* End of CloseUpdateFile */
+} /* End of FinaliseFile */
 
 // destroy nffile handle: free up all resources
 void DisposeFile(nffile_t *nffile) {
@@ -1177,12 +1176,6 @@ void DisposeFile(nffile_t *nffile) {
     if (nffile->ident) free(nffile->ident);
     if (nffile->fileName) free(nffile->fileName);
 
-    queue_close(nffile->processQueue);
-    for (size_t queueLen = queue_length(nffile->processQueue); queueLen > 0; queueLen--) {
-        void *p = queue_pop(nffile->processQueue);
-        FreeDataBlock(p);
-    }
-
     queue_free(nffile->processQueue);
     free(nffile);
 
@@ -1318,7 +1311,7 @@ static dataBlock_t *nfread(nffile_t *nffile) {
 
 }  // End of nfread
 
-__attribute__((noreturn)) void *nfreader(void *arg) {
+__attribute__((noreturn)) static void *nfreader(void *arg) {
     nffile_t *nffile = (nffile_t *)arg;
 
     /* Signal handling */
@@ -1326,10 +1319,9 @@ __attribute__((noreturn)) void *nfreader(void *arg) {
     sigfillset(&set);
     pthread_sigmask(SIG_SETMASK, &set, NULL);
 
-    int terminate = atomic_load(&nffile->terminate);
     int blockCount = 0;
     dataBlock_t *block_header = NULL;
-    while (!terminate && blockCount < nffile->file_header->NumBlocks) {
+    while (blockCount < nffile->file_header->NumBlocks) {
         block_header = nfread(nffile);
         if (!block_header) {
             dbg_printf("block_header == NULL\n");
@@ -1339,19 +1331,12 @@ __attribute__((noreturn)) void *nfreader(void *arg) {
         if (queue_push(nffile->processQueue, (void *)block_header) == QUEUE_CLOSED) {
             FreeDataBlock(block_header);
             dbg_printf("nfreader - processQueue closed\n");
-            terminate = 1;
+            break;
         } else {
             blockCount++;
-            terminate = atomic_load(&nffile->terminate);
             dbg_printf("ReadBlock - expanded: %u\n", block_header->size);
             dbg_printf("Blocks: %u\n", blockCount);
         }
-
-#ifdef DEVEL
-        if (terminate) {
-            printf("Terminate nfreader by signal\n");
-        }
-#endif
     }
 
     // eof or error ends processing
@@ -1360,7 +1345,6 @@ __attribute__((noreturn)) void *nfreader(void *arg) {
     dbg_printf("nfreader done - read %u blocks\n", blockCount);
     dbg_printf("nfreader exit\n");
 
-    atomic_store(&nffile->terminate, 2);
     pthread_exit(NULL);
 
 }  // End of nfreader
@@ -1454,7 +1438,7 @@ static int nfwrite(nffile_t *nffile, dataBlock_t *block_header) {
 
 }  // End of nfwrite
 
-__attribute__((noreturn)) void *nfwriter(void *arg) {
+__attribute__((noreturn)) static void *nfwriter(void *arg) {
     nffile_t *nffile = (nffile_t *)arg;
 
     dbg_printf("nfwriter enter\n");
@@ -1486,11 +1470,7 @@ __attribute__((noreturn)) void *nfwriter(void *arg) {
 
 }  // End of nfwriter
 
-static int SignalTerminate(nffile_t *nffile) {
-    // set terminate
-    atomic_store(&nffile->terminate, 1);
-    queue_close(nffile->processQueue);
-
+static void joinWorkers(nffile_t *nffile) {
     pthread_cond_broadcast(&(nffile->processQueue->cond));
     for (unsigned i = 0; i < NumWorkers; i++) {
         if (nffile->worker[i]) {
@@ -1501,11 +1481,13 @@ static int SignalTerminate(nffile_t *nffile) {
             nffile->worker[i] = 0;
         }
     }
-    atomic_store(&nffile->terminate, 0);
+}  // End of joinWorkers
 
-    return 1;
-
-}  // End of SignalTerminate
+static void TerminateWorkers(nffile_t *nffile) {
+    // closing the queue signals the workers to terminate
+    queue_close(nffile->processQueue);
+    joinWorkers(nffile);
+}  // End of TerminateWorkers
 
 void SetIdent(nffile_t *nffile, char *Ident) {
     if (Ident && strlen(Ident) > 0) {
@@ -1570,6 +1552,32 @@ int ChangeIdent(char *filename, char *Ident) {
 
 }  // End of ChangeIdent
 
+// set unique tmp file name for nfcapd.current.XXXXXX"
+// if
+char *SetUniqueTmpName(char *fname) {
+    size_t len = strlen(fname);
+    if (len < 7) return fname;
+
+    char *c = fname + len - 7;
+    if (*c != '.') return fname;
+    c++;
+
+    // uint32_t randValue = arc4random() & 0xFFFFFF;
+    // need not real pseudo randon number, but different than last time
+    uint32_t randValue = (pseudoID + 1) & 0xFFFFFF;
+    for (int i = 0; i < 6; i++) {
+        uint8_t v = randValue & 0xF;
+        randValue = randValue >> 4;
+        if (v < 10)
+            *c++ = v + '0';
+        else
+            *c++ = v + 'A' - 10;
+    }
+
+    return fname;
+
+}  // End of SetUniqueTmpName
+
 void ModifyCompressFile(int compress) {
     nffile_t *nffile_r, *nffile_w;
     stat_record_t *_s;
@@ -1619,10 +1627,11 @@ void ModifyCompressFile(int compress) {
         }
 
         printf("File %s compression changed\n", nffile_r->fileName);
-        if (!CloseUpdateFile(nffile_w)) {
+        if (!FinaliseFile(nffile_w)) {
+            CloseFile(nffile_w);
             unlink(outfile);
-            LogError("Failed to close file: '%s'", strerror(errno));
         } else {
+            CloseFile(nffile_w);
             if (unlink(nffile_r->fileName)) {
                 LogError("unlink() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
             } else if (rename(outfile, nffile_r->fileName)) {
@@ -2102,8 +2111,8 @@ static void UpdateStat(stat_record_t *s, stat_recordV1_t *sv1) {
     s->numpackets_udp = sv1->numpackets_udp;
     s->numpackets_icmp = sv1->numpackets_icmp;
     s->numpackets_other = sv1->numpackets_other;
-    s->firstseen = 1000LL * (uint64_t)sv1->first_seen + (uint64_t)sv1->msec_first;
-    s->lastseen = 1000LL * (uint64_t)sv1->last_seen + (uint64_t)sv1->msec_last;
+    s->msecFirstSeen = 1000LL * (uint64_t)sv1->first_seen + (uint64_t)sv1->msec_first;
+    s->msecLastSeen = 1000LL * (uint64_t)sv1->last_seen + (uint64_t)sv1->msec_last;
     s->sequence_failure = sv1->sequence_failure;
 }  // End of UpdateStat
 
@@ -2142,10 +2151,10 @@ void PrintStat(stat_record_t *s, char *ident) {
     printf("Bytes_udp: %llu\n", (unsigned long long)s->numbytes_udp);
     printf("Bytes_icmp: %llu\n", (unsigned long long)s->numbytes_icmp);
     printf("Bytes_other: %llu\n", (unsigned long long)s->numbytes_other);
-    printf("First: %llu\n", s->firstseen / 1000LL);
-    printf("Last: %llu\n", s->lastseen / 1000LL);
-    printf("msec_first: %llu\n", s->firstseen % 1000LL);
-    printf("msec_last: %llu\n", s->lastseen % 1000LL);
+    printf("First: %llu\n", s->msecFirstSeen / 1000LL);
+    printf("Last: %llu\n", s->msecLastSeen / 1000LL);
+    printf("msec_first: %llu\n", s->msecFirstSeen % 1000LL);
+    printf("msec_last: %llu\n", s->msecLastSeen % 1000LL);
     printf("Sequence failures: %llu\n", (unsigned long long)s->sequence_failure);
 }  // End of PrintStat
 
@@ -2167,11 +2176,11 @@ void SumStatRecords(stat_record_t *s1, stat_record_t *s2) {
     s1->numpackets_other += s2->numpackets_other;
     s1->sequence_failure += s2->sequence_failure;
 
-    if (s2->firstseen < s1->firstseen) {
-        s1->firstseen = s2->firstseen;
+    if (s2->msecFirstSeen < s1->msecFirstSeen) {
+        s1->msecFirstSeen = s2->msecFirstSeen;
     }
-    if (s2->lastseen > s1->lastseen) {
-        s1->lastseen = s2->lastseen;
+    if (s2->msecLastSeen > s1->msecLastSeen) {
+        s1->msecLastSeen = s2->msecLastSeen;
     }
 
 }  // End of SumStatRecords
