@@ -523,7 +523,7 @@ char *TimeString(uint64_t msecStart, uint64_t msecEnd) {
         char t2[64];
         strftime(t2, 63, "%Y-%m-%d %H:%M:%S", tbuff);
 
-        snprintf(datestr, 254, "%s.%3d - %s.%3d", t1, (int)(msecStart % 1000), t2, (int)(msecEnd % 1000));
+        snprintf(datestr, 254, "%s.%03d - %s.%03d", t1, (int)(msecStart % 1000), t2, (int)(msecEnd % 1000));
     } else {
         snprintf(datestr, 254, "Time Window unknown");
     }
@@ -809,6 +809,8 @@ void DumpHex(FILE *stream, const void *data, size_t size) {
     char ascii[17];
     size_t i, j;
     ascii[16] = '\0';
+    uint32_t addr = 0;
+    fprintf(stream, "%08x ", addr);
     for (i = 0; i < size; ++i) {
         fprintf(stream, "%02X ", ((unsigned char *)data)[i]);
         if (((unsigned char *)data)[i] >= ' ' && ((unsigned char *)data)[i] <= '~') {
@@ -819,7 +821,8 @@ void DumpHex(FILE *stream, const void *data, size_t size) {
         if ((i + 1) % 8 == 0 || i + 1 == size) {
             fprintf(stream, " ");
             if ((i + 1) % 16 == 0) {
-                fprintf(stream, "|  %s \n", ascii);
+                addr += 16;
+                fprintf(stream, "|  %s \n%08x ", ascii, addr);
             } else if (i + 1 == size) {
                 ascii[(i + 1) % 16] = '\0';
                 if ((i + 1) % 16 <= 8) {
