@@ -68,6 +68,8 @@ enum {
     STACK_SECLAST,
     STACK_MSECFIRST,
     STACK_MSECLAST,
+    STACK_DELTAFIRST,
+    STACK_DELTALAST,
     STACK_DURATION,
     STACK_MSEC,
     STACK_SYSUPTIME,
@@ -81,7 +83,7 @@ enum {
  *	which uniquely can identify each exporter/Observation Domain
  */
 typedef struct exporterDomain_s {
-    struct exporterDomain_s *next;  // linkes list to next exporter
+    struct exporterDomain_s *next;  // links list to next exporter
 
     // exporter information
     exporter_info_record_t info;
@@ -213,8 +215,8 @@ static const struct ipfixTranslationMap_s {
     {IPFIX_packetTotalCount, SIZEinPackets, NumberCopy, EXgenericFlowID, OFFinPackets, STACK_NONE, "input packetTotalCount"},
     {IPFIX_flowStartMilliseconds, SIZEmsecFirst, NumberCopy, EXgenericFlowID, OFFmsecFirst, STACK_NONE, "msec first"},
     {IPFIX_flowEndMilliseconds, SIZEmsecLast, NumberCopy, EXgenericFlowID, OFFmsecLast, STACK_NONE, "msec last"},
-    {IPFIX_flowStartDeltaMicroseconds, SIZEmsecFirst, NumberCopy, EXgenericFlowID, OFFmsecFirst, STACK_NONE, "msec first"},
-    {IPFIX_flowEndDeltaMicroseconds, SIZEmsecLast, NumberCopy, EXgenericFlowID, OFFmsecLast, STACK_NONE, "msec last"},
+    {IPFIX_flowStartDeltaMicroseconds, SIZEmsecFirst, NumberCopy, EXgenericFlowID, OFFmsecFirst, STACK_DELTAFIRST, "delta usec first"},
+    {IPFIX_flowEndDeltaMicroseconds, SIZEmsecLast, NumberCopy, EXgenericFlowID, OFFmsecLast, STACK_DELTALAST, "delta usec last"},
     {IPFIX_flowDurationMilliseconds, Stack_ONLY, NumberCopy, EXnull, 0, STACK_DURATION, "duration msec"},
     {LOCAL_IPv4Received, SIZEReceived4IP, NumberCopy, EXipReceivedV4ID, OFFReceived4IP, STACK_NONE, "IPv4 exporter"},
     {LOCAL_IPv6Received, SIZEReceived6IP, NumberCopy, EXipReceivedV6ID, OFFReceived6IP, STACK_NONE, "IPv6 exporter"},
@@ -280,7 +282,7 @@ static const struct ipfixReverseMap_s {
 // module limited globals
 static uint32_t processed_records;
 static int printRecord;
-uint32_t defaultSampling;
+int32_t defaultSampling;
 
 // prototypes
 static void InsertSampler(FlowSource_t *fs, exporterDomain_t *exporter, sampler_record_t *sampler_record);
@@ -1177,7 +1179,7 @@ static void Process_ipfix_option_templates(exporterDomain_t *exporter, void *opt
 
     dbg_printf("\n[%u] Option size: %" PRIu64 ", flags: %" PRIx64 "\n", exporter->info.id, optionTemplate->optionSize, optionTemplate->flags);
     if (optionTemplate->flags) {
-        // if it exitsts - remove old template on exporter with same ID
+        // if it exists - remove old template on exporter with same ID
         templateList_t *template = newTemplate(exporter, tableID);
         if (!template) {
             LogError("Process_ipfix: abort template add: %s line %d", __FILE__, __LINE__);
@@ -1291,6 +1293,7 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
         }
 
         void *outBuff;
+        int redone = 0;
     REDO:
         // map file record to output buffer
         outBuff = GetCurrentCursor(fs->dataBlock);
@@ -1323,7 +1326,7 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
                 }
 
                 // request new and empty buffer
-                LogInfo("Process ipfix: Sequencer run - resize output buffer");
+                dbg_printf("Process ipfix: Sequencer run - resize output buffer\n");
                 // request new and empty buffer
                 fs->dataBlock = WriteBlock(fs->nffile, fs->dataBlock);
                 if (fs->dataBlock == NULL) {
@@ -1337,6 +1340,11 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
                     dbg_printf("Process_ipfix: output buffer size error. Skip ipfix record processing");
                     return;
                 }
+                redone++;
+                if (redone > 2) {
+                    LogError("Process_ipfix: Redone loop - output buffer size error. Skip ipfix record processing");
+                    return;
+                }
                 goto REDO;
                 break;
         }
@@ -1454,10 +1462,13 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
                 dbg_printf("first/last sec abs.\n");
                 genericFlow->msecFirst = stack[STACK_SECFIRST] * (uint64_t)1000;
                 genericFlow->msecLast = stack[STACK_SECLAST] * (uint64_t)1000;
+            } else if (stack[STACK_DELTAFIRST]) {
+                dbg_printf("delta first/last usec.\n");
+                genericFlow->msecFirst = (uint64_t)ExportTime * (uint64_t)1000 - stack[STACK_DELTAFIRST] / (uint64_t)1000;
+                genericFlow->msecLast = (uint64_t)ExportTime * (uint64_t)1000 - stack[STACK_DELTALAST] / (uint64_t)1000;
             }
 
-            if (genericFlow->msecFirst < fs->msecFirst) fs->msecFirst = genericFlow->msecFirst;
-            if (genericFlow->msecLast > fs->msecLast) fs->msecLast = genericFlow->msecLast;
+            UpdateFirstLast(fs->nffile, genericFlow->msecFirst, genericFlow->msecLast);
             dbg_printf("msecFrist: %" PRIu64 "\n", genericFlow->msecFirst);
             dbg_printf("msecLast : %" PRIu64 "\n", genericFlow->msecLast);
             dbg_printf("packets : %" PRIu64 "\n", genericFlow->inPackets);
@@ -1992,7 +2003,7 @@ void Process_IPFIX(void *in_buff, ssize_t in_buff_cnt, FlowSource_t *fs) {
         if (flowset_length == 0) {
             /* 	this should never happen, as 4 is an empty flowset
                     and smaller is an illegal flowset anyway ...
-                    if it happends, we can't determine the next flowset, so skip the entire export
+                    if it happens, we can't determine the next flowset, so skip the entire export
                packet
              */
             LogError("Process_ipfix: flowset zero length error.");
