@@ -99,6 +99,7 @@ static struct aggregationElement_s {
                         {"dstport", {EXgenericFlowID, OFFdstPort, SIZEdstPort, 0}, 0, NOPREPROCESS, 0, 0, "%dp"},
                         {"tos", {EXgenericFlowID, OFFsrcTos, SIZEsrcTos, 0}, 0, NOPREPROCESS, 0, 0, "%tos"},
                         {"srctos", {EXgenericFlowID, OFFsrcTos, SIZEsrcTos, 0}, 0, NOPREPROCESS, 0, 0, "%stos"},
+                        {"flags", {EXgenericFlowID, OFFtcpFlags, SIZEtcpFlags, 0}, 0, NOPREPROCESS, 0, 0, "%flg"},
                         {"srcip4", {EXipv4FlowID, OFFsrc4Addr, SIZEsrc4Addr, AF_INET}, 0, NOPREPROCESS, 0, 1, "%sa"},
                         {"dstip4", {EXipv4FlowID, OFFdst4Addr, SIZEdst4Addr, AF_INET}, 0, NOPREPROCESS, 0, 2, "%da"},
                         {"srcip6", {EXipv6FlowID, OFFsrc6Addr, SIZEsrc6Addr, AF_INET6}, 0, NOPREPROCESS, 0, 1, "%sa"},
@@ -297,7 +298,7 @@ static inline int New_HashKey(void *keymem, recordHandle_t *recordHandle, int sw
  * - record array - static stat record with flow record counters
  *
  * only keep hashValue (32bytes per entry) in hash table.
- * for normal IPv4 aggragation, a key size of 16bytes fit directly into the hashValue. Aggregations up to 16byte
+ * for normal IPv4 aggregation, a key size of 16bytes fit directly into the hashValue. Aggregations up to 16byte
  * hash values profit from fast CPU cache.
  * For aggregation values > 16 bytes, a valPtr points to the nmalloc() value
  */
