@@ -64,6 +64,7 @@ typedef enum {
     IS_NUMBER,
     IS_HEXNUMBER,
     IS_IPADDR,
+    IS_FLAGS,
     IS_MACADDR,
     IS_MPLS_LBL,
     IS_LATENCY,
@@ -90,10 +91,12 @@ static void *SRC_GEO_PreProcess(void *inPtr, recordHandle_t *recordHandle);
 static void *DST_GEO_PreProcess(void *inPtr, recordHandle_t *recordHandle);
 static void *SRC_AS_PreProcess(void *inPtr, recordHandle_t *recordHandle);
 static void *DST_AS_PreProcess(void *inPtr, recordHandle_t *recordHandle);
-static void *JA3_PreProcess(void *inPtr, recordHandle_t *recordHandle);
+static void *JA3in_PreProcess(void *inPtr, recordHandle_t *recordHandle);
+static void *JA3out_PreProcess(void *inPtr, recordHandle_t *recordHandle);
 static void *JA4_PreProcess(void *inPtr, recordHandle_t *recordHandle);
 #ifdef BUILDJA4
-static void *JA4S_PreProcess(void *inPtr, recordHandle_t *recordHandle);
+static void *JA4Sin_PreProcess(void *inPtr, recordHandle_t *recordHandle);
+static void *JA4Sout_PreProcess(void *inPtr, recordHandle_t *recordHandle);
 #endif
 
 typedef struct flow_element_s {
@@ -136,11 +139,12 @@ static struct StatParameter_s {
     {"port", "Port", {EXgenericFlowID, OFFsrcPort, SIZEsrcPort, 0}, IS_NUMBER, NULL},
     {"port", NULL, {EXgenericFlowID, OFFdstPort, SIZEdstPort, 0}, IS_NUMBER, NULL},
     {"proto", "Protocol", {EXgenericFlowID, OFFproto, SIZEproto, 0}, IS_NUMBER, NULL},
+    {"flags", "TCPflags", {EXgenericFlowID, OFFtcpFlags, SIZEtcpFlags, 0}, IS_FLAGS, NULL},
     {"srctos", "Src Tos", {EXgenericFlowID, OFFsrcTos, SIZEsrcTos, 0}, IS_NUMBER, NULL},
     {"dsttos", "Dst Tos", {EXflowMiscID, OFFdstTos, SIZEdstTos, 0}, IS_NUMBER, NULL},
     {"tos", "Tos", {EXgenericFlowID, OFFsrcTos, SIZEsrcTos, 0}, IS_NUMBER, NULL},
     {"tos", NULL, {EXflowMiscID, OFFdstTos, SIZEdstTos, 0}, IS_NUMBER, NULL},
-    {"dir", "Dir", {EXgenericFlowID, OFFdir, SIZEdir, 0}, IS_NUMBER, NULL},
+    {"dir", "Dir", {EXflowMiscID, OFFdir, SIZEdir, 0}, IS_NUMBER, NULL},
     {"srcas", "Src AS", {EXasRoutingID, OFFsrcAS, SIZEsrcAS, 0}, IS_NUMBER, SRC_AS_PreProcess},
     {"srcasn", "                                 Organisation (AS num)", {EXasRoutingID, OFFsrcAS, SIZEsrcAS, 0}, IS_ASORG, SRC_AS_PreProcess},
     {"dstas", "Dst AS", {EXasRoutingID, OFFdstAS, SIZEdstAS, 0}, IS_NUMBER, DST_AS_PreProcess},
@@ -159,6 +163,8 @@ static struct StatParameter_s {
     {"dstmask", "Dst Mask", {EXflowMiscID, OFFdstMask, SIZEdstMask, 0}, IS_NUMBER, NULL},
     {"mask", "Mask", {EXflowMiscID, OFFsrcMask, SIZEsrcMask, 0}, IS_NUMBER, NULL},
     {"mask", NULL, {EXflowMiscID, OFFdstMask, SIZEdstMask, 0}, IS_NUMBER, NULL},
+    {"tunproto", "Tun proto", {EXtunIPv4ID, OFFtunProtoV4, SIZEtunProtoV4, 0}, IS_NUMBER, NULL},
+    {"tunproto", NULL, {EXtunIPv6ID, OFFtunProtoV6, SIZEtunProtoV6, 0}, IS_NUMBER, NULL},
     {"srcvlan", "Src Vlan", {EXvLanID, OFFvlanID, SIZEvlanID, 0}, IS_NUMBER, NULL},
     {"dstvlan", "Dst Vlan", {EXvLanID, OFFpostVlanID, SIZEpostVlanID, 0}, IS_NUMBER, NULL},
     {"vlan", "Vlan", {EXvLanID, OFFvlanID, SIZEvlanID, 0}, IS_NUMBER, NULL},
@@ -194,10 +200,12 @@ static struct StatParameter_s {
     {"sl", "Server Latency", {EXlatencyID, OFFusecServerNwDelay, SIZEusecServerNwDelay, 0}, IS_LATENCY, NULL},
     {"al", "Application Latency", {EXlatencyID, OFFusecApplLatency, SIZEusecApplLatency, 0}, IS_LATENCY, NULL},
     {"nbar", "Nbar", {EXnbarAppID, OFFnbarAppID, SIZEnbarAppID, 0}, IS_NBAR, NULL},
-    {"ja3", "ja3                             ", {JA3index, OFFja3String, SIZEja3String + 1, 0}, IS_JA3, JA3_PreProcess},
-    {"ja4", "ja4                                ", {JA4index, OFFja4String, SIZEja4String + 1, 0}, IS_JA4, JA4_PreProcess},
+    {"ja3", "ja3                             ", {EXinPayloadHandle, OFFja3String, SIZEja3String + 1, 0}, IS_JA3, JA3in_PreProcess},
+    {"ja3", NULL, {EXoutPayloadHandle, OFFja3String, SIZEja3String + 1, 0}, IS_JA3, JA3out_PreProcess},
+    {"ja4", "ja4                                ", {EXinPayloadHandle, OFFja4String, SIZEja4String + 1, 0}, IS_JA4, JA4_PreProcess},
 #ifdef BUILDJA4
-    {"ja4s", "ja4s                    ", {JA4index, OFFja4String, SIZEja4sString + 1, 0}, IS_JA4S, JA4S_PreProcess},
+    {"ja4s", "ja4s                    ", {EXinPayloadHandle, OFFja4String, SIZEja4sString + 1, 0}, IS_JA4S, JA4Sin_PreProcess},
+    {"ja4s", NULL, {EXoutPayloadHandle, OFFja4String, SIZEja4sString + 1, 0}, IS_JA4S, JA4Sout_PreProcess},
 #endif
     {"odid", "Obs DomainID", {EXobservationID, OFFdomainID, SIZEdomainID, 0}, IS_HEXNUMBER, NULL},
     {"opid", "Obs PointID", {EXobservationID, OFFpointID, SIZEpointID, 0}, IS_HEXNUMBER, NULL},
@@ -645,6 +653,7 @@ int SetElementStat(char *elementStat, char *orderBy) {
     }
 
     if (strcasecmp(elementStat, "proto") == 0) request->order_proto = 1;
+    if (strcasecmp(elementStat, "tunproto") == 0) request->order_proto = 2;
 
     int i = 0;
     while (StatParameters[i].statname) {
@@ -692,9 +701,9 @@ static inline void *DST_GEO_PreProcess(void *inPtr, recordHandle_t *recordHandle
     char *geo = (char *)inPtr + OFFgeoDstIP;
     if (HasGeoDB == 0 || geo[0]) return inPtr;
     if (ipv4Flow)
-        LookupV4Country(ipv4Flow->dstAddr, inPtr);
+        LookupV4Country(ipv4Flow->dstAddr, geo);
     else if (ipv6Flow)
-        LookupV6Country(ipv6Flow->dstAddr, inPtr);
+        LookupV6Country(ipv6Flow->dstAddr, geo);
 
     return inPtr;
 }  // End of DST_GEO_PreProcess
@@ -735,26 +744,77 @@ static inline void *DST_AS_PreProcess(void *inPtr, recordHandle_t *recordHandle)
     return inPtr;
 }  // End of DST_AS_PreProcess
 
-static inline void *JA3_PreProcess(void *inPtr, recordHandle_t *recordHandle) {
+static inline void *JA3in_PreProcess(void *inPtr, recordHandle_t *recordHandle) {
     EXgenericFlow_t *genericFlow = (EXgenericFlow_t *)recordHandle->extensionList[EXgenericFlowID];
     const uint8_t *payload = (const uint8_t *)recordHandle->extensionList[EXinPayloadID];
     if (payload == NULL || genericFlow->proto != IPPROTO_TCP || inPtr) return inPtr;
 
-    ssl_t *ssl = recordHandle->extensionList[SSLindex];
+    payloadHandle_t *payloadHandle = (payloadHandle_t *)recordHandle->extensionList[EXinPayloadHandle];
+    if (payloadHandle == NULL) {
+        payloadHandle = calloc(1, sizeof(payloadHandle_t));
+        if (!payloadHandle) {
+            LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
+            exit(EXIT_FAILURE);
+        } else {
+            recordHandle->extensionList[EXinPayloadHandle] = payloadHandle;
+        }
+    }
+
+    ssl_t *ssl = payloadHandle->ssl;
     if (ssl == NULL) {
         uint32_t payloadLength = ExtensionLength(payload);
         ssl = sslProcess(payload, payloadLength);
-        recordHandle->extensionList[SSLindex] = ssl;
+        payloadHandle->ssl = ssl;
         if (ssl == NULL) {
             return NULL;
         }
     }
+
     // ssl is defined
-    char *ja3 = ja3Process(ssl, NULL);
-    recordHandle->extensionList[JA3index] = ja3;
+    char *ja3 = payloadHandle->ja3;
+    if (payloadHandle->ja3 == NULL) {
+        ja3 = ja3Process(ssl, NULL);
+        payloadHandle->ja3 = ja3;
+    }
     return ja3;
 
-}  // End of JA3_PreProcess
+}  // End of JA3in_PreProcess
+
+static inline void *JA3out_PreProcess(void *inPtr, recordHandle_t *recordHandle) {
+    EXgenericFlow_t *genericFlow = (EXgenericFlow_t *)recordHandle->extensionList[EXgenericFlowID];
+    const uint8_t *payload = (const uint8_t *)recordHandle->extensionList[EXoutPayloadID];
+    if (payload == NULL || genericFlow->proto != IPPROTO_TCP || inPtr) return inPtr;
+
+    payloadHandle_t *payloadHandle = (payloadHandle_t *)recordHandle->extensionList[EXoutPayloadHandle];
+    if (payloadHandle == NULL) {
+        payloadHandle = calloc(1, sizeof(payloadHandle_t));
+        if (!payloadHandle) {
+            LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
+            exit(EXIT_FAILURE);
+        } else {
+            recordHandle->extensionList[EXoutPayloadHandle] = payloadHandle;
+        }
+    }
+
+    ssl_t *ssl = payloadHandle->ssl;
+    if (ssl == NULL) {
+        uint32_t payloadLength = ExtensionLength(payload);
+        ssl = sslProcess(payload, payloadLength);
+        payloadHandle->ssl = ssl;
+        if (ssl == NULL) {
+            return NULL;
+        }
+    }
+
+    // ssl is defined
+    char *ja3 = payloadHandle->ja3;
+    if (payloadHandle->ja3 == NULL) {
+        ja3 = ja3Process(ssl, NULL);
+        payloadHandle->ja3 = ja3;
+    }
+    return ja3;
+
+}  // End of JA3out_PreProcess
 
 static inline void *JA4_PreProcess(void *inPtr, recordHandle_t *recordHandle) {
     EXgenericFlow_t *genericFlow = (EXgenericFlow_t *)recordHandle->extensionList[EXgenericFlowID];
@@ -762,66 +822,129 @@ static inline void *JA4_PreProcess(void *inPtr, recordHandle_t *recordHandle) {
     EXinPayload_t *payload = (EXinPayload_t *)recordHandle->extensionList[EXinPayloadID];
     if (payload == NULL || genericFlow->proto != IPPROTO_TCP) return NULL;
 
-    ssl_t *ssl = recordHandle->extensionList[SSLindex];
+    payloadHandle_t *payloadHandle = (payloadHandle_t *)recordHandle->extensionList[EXinPayloadHandle];
+    if (payloadHandle == NULL) {
+        payloadHandle = calloc(1, sizeof(payloadHandle_t));
+        if (!payloadHandle) {
+            LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
+            exit(EXIT_FAILURE);
+        } else {
+            recordHandle->extensionList[EXinPayloadHandle] = payloadHandle;
+        }
+    }
+
+    ssl_t *ssl = payloadHandle->ssl;
     if (ssl == NULL) {
         uint32_t payloadLength = ExtensionLength(payload);
         ssl = sslProcess(payload, payloadLength);
-        recordHandle->extensionList[SSLindex] = ssl;
+        payloadHandle->ssl = ssl;
         if (ssl == NULL) {
             return NULL;
         }
     }
     // ssl is defined
-    ja4_t *ja4 = NULL;
-    if (ssl->type == CLIENTssl) {
+    if (ssl->type != CLIENTssl) return NULL;
+
+    ja4_t *ja4 = payloadHandle->ja4;
+    if (ja4 == NULL) {
         ja4 = ja4Process(ssl, genericFlow->proto);
-    } else {
-        return NULL;
+        payloadHandle->ja4 = ja4;
     }
 
-    recordHandle->extensionList[JA4index] = ja4;
     return ja4;
 
 }  // End of JA4_PreProcess
 
 #ifdef BUILDJA4
-static inline void *JA4S_PreProcess(void *inPtr, recordHandle_t *recordHandle) {
+static inline void *JA4Sin_PreProcess(void *inPtr, recordHandle_t *recordHandle) {
     EXgenericFlow_t *genericFlow = (EXgenericFlow_t *)recordHandle->extensionList[EXgenericFlowID];
 
     EXinPayload_t *payload = (EXinPayload_t *)recordHandle->extensionList[EXinPayloadID];
     if (payload == NULL || genericFlow->proto != IPPROTO_TCP) return NULL;
 
-    ssl_t *ssl = recordHandle->extensionList[SSLindex];
+    payloadHandle_t *payloadHandle = (payloadHandle_t *)recordHandle->extensionList[EXinPayloadHandle];
+    if (payloadHandle == NULL) {
+        payloadHandle = calloc(1, sizeof(payloadHandle_t));
+        if (!payloadHandle) {
+            LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
+            exit(EXIT_FAILURE);
+        } else {
+            recordHandle->extensionList[EXinPayloadHandle] = payloadHandle;
+        }
+    }
+
+    ssl_t *ssl = payloadHandle->ssl;
     if (ssl == NULL) {
         uint32_t payloadLength = ExtensionLength(payload);
         ssl = sslProcess(payload, payloadLength);
-        recordHandle->extensionList[SSLindex] = ssl;
+        payloadHandle->ssl = ssl;
         if (ssl == NULL) {
             return NULL;
         }
     }
-    // ssl is defined
-    ja4_t *ja4 = NULL;
-    if (ssl->type == SERVERssl) {
-        ja4 = ja4sProcess(ssl, genericFlow->proto);
-    } else {
-        return NULL;
-    }
 
-    recordHandle->extensionList[JA4index] = ja4;
+    // ssl is defined
+    if (ssl->type != SERVERssl) return NULL;
+
+    ja4_t *ja4 = payloadHandle->ja4;
+    if (ja4 == NULL) {
+        ja4 = ja4sProcess(ssl, genericFlow->proto);
+        payloadHandle->ja4 = ja4;
+    }
     return ja4;
 
-}  // End of JA4S_PreProcess
+}  // End of JA4Sin_PreProcess
+
+static inline void *JA4Sout_PreProcess(void *inPtr, recordHandle_t *recordHandle) {
+    EXgenericFlow_t *genericFlow = (EXgenericFlow_t *)recordHandle->extensionList[EXgenericFlowID];
+
+    EXinPayload_t *payload = (EXinPayload_t *)recordHandle->extensionList[EXoutPayloadID];
+    if (payload == NULL || genericFlow->proto != IPPROTO_TCP) return NULL;
+
+    payloadHandle_t *payloadHandle = (payloadHandle_t *)recordHandle->extensionList[EXoutPayloadHandle];
+    if (payloadHandle == NULL) {
+        payloadHandle = calloc(1, sizeof(payloadHandle_t));
+        if (!payloadHandle) {
+            LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
+            exit(EXIT_FAILURE);
+        } else {
+            recordHandle->extensionList[EXoutPayloadHandle] = payloadHandle;
+        }
+    }
+
+    ssl_t *ssl = payloadHandle->ssl;
+    if (ssl == NULL) {
+        uint32_t payloadLength = ExtensionLength(payload);
+        ssl = sslProcess(payload, payloadLength);
+        payloadHandle->ssl = ssl;
+        if (ssl == NULL) {
+            return NULL;
+        }
+    }
+
+    // ssl is defined
+    if (ssl->type != SERVERssl) return NULL;
+
+    ja4_t *ja4 = payloadHandle->ja4;
+    if (ja4 == NULL) {
+        ja4 = ja4sProcess(ssl, genericFlow->proto);
+        payloadHandle->ja4 = ja4;
+    }
+    return ja4;
+
+}  // End of JA4Sout_PreProcess
+
 #endif
 
 void AddElementStat(recordHandle_t *recordHandle) {
     EXgenericFlow_t *genericFlow = (EXgenericFlow_t *)recordHandle->extensionList[EXgenericFlowID];
     if (!genericFlow) return;
+    uint64_t zero = 0;
 
     // for every requested -s stat do
     for (int i = 0; i < NumStats; i++) {
         hashkey_t hashkey = {0};
-        hashkey.proto = StatRequest[i].order_proto ? genericFlow->proto : 0;
+        hashkey.proto = StatRequest[i].order_proto == 1 ? genericFlow->proto : 0;
         int index = StatRequest[i].StatType;
         // for the number of elements in this stat type
         do {
@@ -834,8 +957,14 @@ void AddElementStat(recordHandle_t *recordHandle) {
                 inPtr = preproc(inPtr, recordHandle);
             }
             if (inPtr == NULL) {
-                index++;
-                continue;
+                int type = StatParameters[index].type;
+                if (type == IS_NUMBER) {
+                    inPtr = &zero;
+                    offset = 0;
+                } else {
+                    index++;
+                    continue;
+                }
             }
             inPtr += offset;
 
@@ -950,6 +1079,12 @@ static void PrintStatLine(stat_record_t *stat, outputParams_t *outputParams, Sor
                 }
             }
             break;
+        case IS_FLAGS:
+            if (outputParams->printPlain)
+                snprintf(valstr, 64, "0x%" PRIx64, hashKey->v1);
+            else
+                snprintf(valstr, 64, "%8s", FlagsString(hashKey->v1));
+            break;
         case IS_MACADDR: {
             uint8_t mac[6];
             for (int i = 0; i < 6; i++) {
@@ -1013,11 +1148,11 @@ static void PrintStatLine(stat_record_t *stat, outputParams_t *outputParams, Sor
         } break;
         case IS_GEO: {
             snprintf(valstr, 64, "%s", (char *)&(hashKey->v1));
-        }
+        } break;
         case IS_ASORG: {
             const char *org = LookupASorg(hashKey->v1);
             snprintf(valstr, 64, "%45s (%6" PRIi64 ")", org != NULL ? org : "unknown", hashKey->v1);
-        }
+        } break;
     }
     valstr[63] = 0;
 
@@ -1084,7 +1219,17 @@ static void PrintStatLine(stat_record_t *stat, outputParams_t *outputParams, Sor
     char datestr[64];
     strftime(datestr, 63, "%Y-%m-%d %H:%M:%S", tbuff);
 
-    char *protoStr = order_proto ? ProtoString(hashKey->proto, outputParams->printPlain) : "any";
+    char *protoStr;
+    switch (order_proto) {
+        case 1:
+            protoStr = ProtoString(hashKey->proto, outputParams->printPlain);
+            break;
+        case 2:
+            protoStr = ProtoString(hashKey->v1, outputParams->printPlain);
+            break;
+        default:
+            protoStr = "any";
+    }
     char dStr[64];
     if (outputParams->printPlain)
         snprintf(dStr, 64, "%16.3f", duration);
@@ -1092,9 +1237,8 @@ static void PrintStatLine(stat_record_t *stat, outputParams_t *outputParams, Sor
         snprintf(dStr, 64, "%s", DurationString(duration));
 
     if (Getv6Mode() && (type == IS_IPADDR)) {
-        printf("%s.%03u %9.3f %-5s %s%39s %8s(%4.1f) %8s(%4.1f) %8s(%4.1f) %8s %8s %5u\n", datestr, (unsigned)(statRecord->msecFirst % 1000),
-               duration, protoStr, tag_string, valstr, flows_str, flows_percent, packets_str, packets_percent, byte_str, bytes_percent, pps_str,
-               bps_str, bpp);
+        printf("%s.%03u %9s %-5s %s%43s %8s(%4.1f) %8s(%4.1f) %8s(%4.1f) %8s %8s %5u\n", datestr, (unsigned)(statRecord->msecFirst % 1000), dStr,
+               protoStr, tag_string, valstr, flows_str, flows_percent, packets_str, packets_percent, byte_str, bytes_percent, pps_str, bps_str, bpp);
     } else {
         if (outputParams->hasGeoDB) {
             printf("%s.%03u %9s %-5s %s%21s %8s(%4.1f) %8s(%4.1f) %8s(%4.1f) %8s %8s %5u\n", datestr, (unsigned)(statRecord->msecFirst % 1000), dStr,
@@ -1391,7 +1535,7 @@ void PrintElementStat(stat_record_t *sum_stat, outputParams_t *outputParams, Rec
                     }
                     if (Getv6Mode() && (type == IS_IPADDR)) {
                         printf(
-                            "Date first seen             Duration     Proto %39s    Flows(%%)     Packets(%%)       Bytes(%%)         pps      "
+                            "Date first seen             Duration     Proto %43s    Flows(%%)     Packets(%%)       Bytes(%%)         pps      "
                             "bps   "
                             "bpp\n",
                             StatParameters[stat].HeaderInfo);
@@ -1481,13 +1625,12 @@ static SortElement_t *StatTopN(int topN, uint32_t *count, int hash_num, int orde
     uint32_t c = 0;
     for (uint32_t i = 0; i < elemenHash->capacity; i++) {
         if (elemenHash->keys[i].active) {
+            dbg_printf("Get next active hashCell. count: %" PRIu64 "\n", topN_list[c].count);
             StatRecord_t *record = &(elemenHash->records[i]);
             topN_list[c].count = orderByTable[order].element_function(record);
             topN_list[c].record = (void *)record;
             record->hashkey = &(elemenHash->keys[i].key);
             c++;
-
-            dbg_printf("Get next hashCell. count: %" PRIu64 "\n", topN_list[c].count);
         }
     }
     assert(c == numCells);
