@@ -64,6 +64,7 @@
 #include "bookkeeper.h"
 #include "collector.h"
 #include "flowtree.h"
+#include "ip_frag.h"
 #include "nfdump.h"
 #include "nffile.h"
 #include "nflog.h"
@@ -282,85 +283,6 @@ void SetApplication_latency(struct FlowNode *node, struct timeval *t_packet) {
 
 }  // End of SetApplication_latency
 
-static inline struct FlowNode *ProcessIPfrag(packetParam_t *packetParam, const struct pcap_pkthdr *hdr, struct ip *ip, void *eodata) {
-    uint16_t ip_off = ntohs(ip->ip_off);
-    uint32_t frag_offset = (ip_off & IP_OFFMASK) << 3;
-    int size_ip = (ip->ip_hl << 2);
-
-    struct FlowNode *Node = NULL;
-    if (frag_offset == 0) {
-        // first fragment in sequence
-        dbg_printf("Fragmented packet: first segment: ip_off: %u, frag_offset: %u\n", ip_off, frag_offset);
-        Node = New_Node();
-        Node->t_first.tv_sec = hdr->ts.tv_sec;
-        Node->t_first.tv_usec = hdr->ts.tv_usec;
-        Node->t_last.tv_sec = hdr->ts.tv_sec;
-        Node->t_last.tv_usec = hdr->ts.tv_usec;
-        Node->flowKey.version = AF_INET;
-        Node->flowKey.proto = ip->ip_p;
-        Node->flowKey.src_addr.v4 = ntohl(ip->ip_src.s_addr);
-        Node->flowKey.dst_addr.v4 = ntohl(ip->ip_dst.s_addr);
-        Node->flowKey.src_port = ntohs(ip->ip_id);
-        Node->flowKey.dst_port = 0;
-        Node->nodeType = FRAG_NODE;
-
-        if (Insert_Node(Node) != NULL) {
-            dbg_printf("IP fragment: initial node already exists! Skip!\n");
-            Free_Node(Node);
-            return NULL;
-        }
-
-        // allocate enough memory for udp packet
-        Node->payload = calloc(1, 65536);
-        if (!Node->payload) {
-            LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
-            Remove_Node(Node);
-            Free_Node(Node);
-            return NULL;
-        }
-    } else {
-        struct FlowNode FindNode = {0};
-        FindNode.flowKey.version = AF_INET;
-        FindNode.flowKey.proto = ip->ip_p;
-        FindNode.flowKey.src_addr.v4 = ntohl(ip->ip_src.s_addr);
-        FindNode.flowKey.dst_addr.v4 = ntohl(ip->ip_dst.s_addr);
-        FindNode.flowKey.src_port = ntohs(ip->ip_id);
-        FindNode.flowKey.dst_port = 0;
-
-        Node = Lookup_Node(&FindNode);
-        if (!Node || Node->nodeType != FRAG_NODE) {
-            dbg_printf("IP fragment: initial node missing! Skip!\n");
-            packetParam->proc_stat.skipped++;
-            return NULL;
-        }
-
-        if ((ip_off & IP_MF) && frag_offset) dbg_printf("Fragmented packet: middle segment: ip_off: %u, frag_offset: %u\n", ip_off, frag_offset);
-    }
-
-    void *dataptr = (void *)ip + size_ip;
-    ptrdiff_t len = eodata - dataptr;
-    dbg_printf("IP frag: Insert fragment at offset: %u, length: %td\n", frag_offset, len);
-    if ((frag_offset + len) > 65536) {
-        LogError("IP fragmen too large: %.", frag_offset + len);
-        Remove_Node(Node);
-        Free_Node(Node);
-        return NULL;
-    }
-
-    memcpy(Node->payload + frag_offset, dataptr, len);
-
-    if ((ip_off & IP_MF) == 0) {
-        // last fragment - export node
-        Node->payloadSize = frag_offset + len;
-        Node->bytes = size_ip + Node->payloadSize;
-        dbg_printf("Fragmented packet: last segment: ip_off: %u, frag_offset: %u, total len: %u\n", ip_off, frag_offset, Node->payloadSize);
-        Remove_Node(Node);
-        return Node;
-    }
-
-    return NULL;
-}  // End of ProcessIPfrag
-
 static inline void AddPayload(struct FlowNode *Node, void *payload, size_t payloadSize) {
     Node->payload = malloc(payloadSize);
     if (!Node->payload) {
@@ -428,7 +350,7 @@ static inline void ProcessTCPFlow(packetParam_t *packetParam, struct FlowNode *N
     if (NewNode->minTTL < Node->minTTL) Node->minTTL = NewNode->minTTL;
     if (NewNode->maxTTL > Node->maxTTL) Node->maxTTL = NewNode->maxTTL;
 
-        // DEVEL RTT - disabled for now
+    // DEVEL RTT - disabled for now
 #if 0
     if (NewNode->signal != SIGNAL_FIN && Node->latency.ack && ((NewNode->latency.ack - Node->latency.ack) > 0)) {
         uint32_t rtt = NewNode->latency.tsVal - Node->latency.tsVal;
@@ -562,7 +484,7 @@ int ProcessPacket(packetParam_t *packetParam, const struct pcap_pkthdr *hdr, con
     uint8_t *eodata = (uint8_t *)data + hdr->caplen;
     void *defragmented = NULL;
     void *payload = NULL;
-    size_t payloadSize = 0;
+    ssize_t payloadSize = 0;
     uint32_t vlanID = 0;
     uint64_t srcMac = 0;
     uint64_t dstMac = 0;
@@ -827,6 +749,8 @@ REDO_IPPROTO:
     struct ip *ip = (struct ip *)dataptr;  // offset points to end of link layer
     version = ip->ip_v;                    // ip version
 
+    ptrdiff_t ipPayloadLength = 0;
+    uint8_t *ipPayloadEnd = NULL;
     if (version == 6) {
         struct ip6_hdr *ip6 = (struct ip6_hdr *)dataptr;
         size_t size_ip = sizeof(struct ip6_hdr);
@@ -839,7 +763,7 @@ REDO_IPPROTO:
         }
 
         // IPv6 duplicate check
-        // dublicate check starts from the IP header over the rest of the packet
+        // duplicate check starts from the IP header over the rest of the packet
         // vlan, mpls and layer 1 headers are ignored
         if (unlikely(packetParam->doDedup && redoLink == 0)) {
             // check for de-dup
@@ -857,8 +781,28 @@ REDO_IPPROTO:
 
         // ipv6 Extension headers not processed
         IPproto = ip6->ip6_ctlun.ip6_un1.ip6_un1_nxt;
-        dbg_printf("Packet IPv6, SRC %s, DST %s\n", inet_ntop(AF_INET6, &ip6->ip6_src, s1, sizeof(s1)),
-                   inet_ntop(AF_INET6, &ip6->ip6_dst, s2, sizeof(s2)));
+        struct ip6_frag *ip6_frag = NULL;
+        uint8_t fragment_flag = 0;
+        if (unlikely(IPproto == IPPROTO_FRAGMENT)) {
+            ip6_frag = (struct ip6_frag *)dataptr;
+            IPproto = ip6_frag->ip6f_nxt;
+            void *payload = ProcessIP6Fragment(ip6, ip6_frag, eodata);
+            if (payload == NULL) {
+                // not yet complete
+                dbg_printf("IPv6 de-fragmentation not yet completed\n");
+                goto END_FUNC;
+            }
+            defragmented = payload;
+            dataptr = payload;
+            ipPayloadLength = ntohs(ip6->ip6_ctlun.ip6_un1.ip6_un1_plen) - sizeof(struct ip6_frag);
+            eodata = dataptr + ipPayloadLength;
+            fragment_flag = flagMF;
+        } else {
+            ipPayloadLength = ntohs(ip6->ip6_ctlun.ip6_un1.ip6_un1_plen);
+            dbg_printf("Packet IPv6, SRC %s, DST %s, padding %zu\n", inet_ntop(AF_INET6, &ip6->ip6_src, s1, sizeof(s1)),
+                       inet_ntop(AF_INET6, &ip6->ip6_dst, s2, sizeof(s2)), (ptrdiff_t)(eodata - ipPayloadEnd));
+        }
+        ipPayloadEnd = dataptr + ipPayloadLength;
 
         if (!Node) Node = New_Node();
         Node->flowKey.version = AF_INET6;
@@ -870,22 +814,12 @@ REDO_IPPROTO:
         uint8_t ttl = ip6->ip6_ctlun.ip6_un1.ip6_un1_hlim;
         Node->minTTL = ttl;
         Node->maxTTL = ttl;
-        Node->fragmentFlags = 0;
+        Node->fragmentFlags = fragment_flag;
 
-        // keep compiler happy - gets optimized out anyway
-        void *p = (void *)&ip6->ip6_src;
-        uint64_t *addr = (uint64_t *)p;
-        Node->flowKey.src_addr.v6[0] = ntohll(addr[0]);
-        Node->flowKey.src_addr.v6[1] = ntohll(addr[1]);
-
-        p = (void *)&ip6->ip6_dst;
-        addr = (uint64_t *)p;
-        Node->flowKey.dst_addr.v6[0] = ntohll(addr[0]);
-        Node->flowKey.dst_addr.v6[1] = ntohll(addr[1]);
+        memcpy(Node->flowKey.src_addr.V6, ip6->ip6_src.s6_addr, 16);
+        memcpy(Node->flowKey.dst_addr.V6, ip6->ip6_dst.s6_addr, 16);
 
     } else if (version == 4) {
-        uint16_t ip_off = ntohs(ip->ip_off);
-        uint32_t frag_offset = (ip_off & IP_OFFMASK) << 3;
         int size_ip = (ip->ip_hl << 2);
 
         dataptr += size_ip;
@@ -895,9 +829,13 @@ REDO_IPPROTO:
             goto END_FUNC;
         }
 
+        ipPayloadLength = ntohs(ip->ip_len) - size_ip;
+        ipPayloadEnd = dataptr + ipPayloadLength;
+
         // IPv4 duplicate check
-        // dublicate check starts from the IP header over the rest of the packet
+        // duplicate check starts from the IP header over the rest of the packet
         // vlan, mpls and layer 1 headers are ignored
+        uint8_t fragment_flag = 0;
         if (unlikely(packetParam->doDedup && redoLink == 0)) {
             // check for de-dup
             uint32_t ttl = ip->ip_ttl;
@@ -915,40 +853,47 @@ REDO_IPPROTO:
         }
 
         IPproto = ip->ip_p;
-        dbg_printf("Packet IPv4 SRC %s, DST %s\n", inet_ntop(AF_INET, &ip->ip_src, s1, sizeof(s1)), inet_ntop(AF_INET, &ip->ip_dst, s2, sizeof(s2)));
+        dbg_printf("Packet IPv4 SRC %s, DST %s, padding %zu\n", inet_ntop(AF_INET, &ip->ip_src, s1, sizeof(s1)),
+                   inet_ntop(AF_INET, &ip->ip_dst, s2, sizeof(s2)), (ptrdiff_t)(eodata - ipPayloadEnd));
 
         // IPv4 defragmentation
+        uint16_t ip_off = ntohs(ip->ip_off);
+        uint32_t frag_offset = (ip_off & IP_OFFMASK) << 3;
         if ((ip_off & IP_MF) || frag_offset) {
             // fragmented packet
-            Node = ProcessIPfrag(packetParam, hdr, ip, eodata);
-            if (Node == NULL) {
+            void *payload = ProcessIP4Fragment(ip, eodata);
+            if (payload == NULL) {
                 // not yet complete
-                dbg_printf("Fragmentation not yet completed. Size %td bytes\n", eodata - dataptr);
+                dbg_printf("IPv4 de-fragmentation not yet completed\n");
                 goto END_FUNC;
             }
-            dbg_printf("Fragmentation complete: %u bytes\n", Node->bytes);
-            // packet defragmented - set payload to defragmented data
-            defragmented = Node->payload;
-            dataptr = Node->payload;
-            eodata = dataptr + Node->payloadSize;
-            Node->payload = NULL;
-            Node->payloadSize = 0;
-            Node->fragmentFlags |= flagMF;
-        } else {
-            if (!Node) Node = New_Node();
-            Node->flowKey.version = AF_INET;
-            Node->t_first.tv_sec = hdr->ts.tv_sec;
-            Node->t_last.tv_sec = hdr->ts.tv_sec;
-            Node->t_first.tv_usec = hdr->ts.tv_usec;
-            Node->t_last.tv_usec = hdr->ts.tv_usec;
-            Node->bytes = ntohs(ip->ip_len);
-            if (ip_off & IP_DF) Node->fragmentFlags |= flagDF;
 
-            Node->flowKey.src_addr.v4 = ntohl(ip->ip_src.s_addr);
-            Node->flowKey.dst_addr.v4 = ntohl(ip->ip_dst.s_addr);
+            // packet defragmented - set payload to defragmented data
+            defragmented = payload;
+            dataptr = payload;
+            ipPayloadLength = ntohs(ip->ip_len) - size_ip;
+            eodata = dataptr + ipPayloadLength;
+            fragment_flag = flagMF;
+        } else {
+            ipPayloadLength = ntohs(ip->ip_len) - size_ip;
         }
+        ipPayloadEnd = dataptr + ipPayloadLength;
+
+        if (!Node) Node = New_Node();
+        Node->flowKey.version = AF_INET;
+        Node->t_first.tv_sec = hdr->ts.tv_sec;
+        Node->t_last.tv_sec = hdr->ts.tv_sec;
+        Node->t_first.tv_usec = hdr->ts.tv_usec;
+        Node->t_last.tv_usec = hdr->ts.tv_usec;
+        Node->bytes = ntohs(ip->ip_len);
+
+        Node->flowKey.src_addr.v4 = ntohl(ip->ip_src.s_addr);
+        Node->flowKey.dst_addr.v4 = ntohl(ip->ip_dst.s_addr);
         Node->minTTL = ip->ip_ttl;
         Node->maxTTL = ip->ip_ttl;
+        Node->fragmentFlags = fragment_flag;
+        if (ip_off & IP_DF) Node->fragmentFlags |= flagDF;
+
     } else {
         dbg_printf("ProcessPacket() Unsupported protocol version: %i\n", version);
         packetParam->proc_stat.unknown++;
@@ -975,6 +920,12 @@ REDO_IPPROTO:
         }
     }
 
+    if (ipPayloadEnd < dataptr || ipPayloadEnd > eodata) {
+        LogError("ProcessPacket() payload data length error line: %u", __LINE__);
+        Free_Node(Node);
+        goto END_FUNC;
+    }
+
     // transport protocol processing
     switch (IPproto) {
         case IPPROTO_UDP: {
@@ -1003,7 +954,7 @@ REDO_IPPROTO:
             Node->flowKey.dst_port = ntohs(udp->uh_dport);
 
             dbg_assert(dataptr <= eodata);
-            payloadSize = (ptrdiff_t)(eodata - dataptr);
+            payloadSize = (ptrdiff_t)(ipPayloadEnd - dataptr);
             if (payloadSize > 0) payload = (void *)dataptr;
             ProcessUDPFlow(packetParam, Node, payload, payloadSize);
 
@@ -1012,7 +963,6 @@ REDO_IPPROTO:
             struct tcphdr *tcp = (struct tcphdr *)dataptr;
             uint32_t size_tcp = tcp->th_off << 2;
             dataptr += size_tcp;
-
             if (dataptr > eodata) {
                 dbg_printf("  TCP Short packet: %u, Check line: %u\n", hdr->caplen, __LINE__);
                 packetParam->proc_stat.short_snap++;
@@ -1021,7 +971,7 @@ REDO_IPPROTO:
             }
 
             dbg_assert(dataptr <= eodata);
-            payloadSize = (ptrdiff_t)(eodata - dataptr);
+            payloadSize = (ptrdiff_t)(ipPayloadEnd - dataptr);
             if (payloadSize > 0) payload = (void *)dataptr;
 
 #ifdef DEVEL
@@ -1096,7 +1046,7 @@ REDO_IPPROTO:
             }
 
             dbg_assert(dataptr <= eodata);
-            payloadSize = (ptrdiff_t)(eodata - dataptr);
+            payloadSize = (ptrdiff_t)(ipPayloadEnd - dataptr);
             if (payloadSize > 0) payload = (void *)dataptr;
 
             Node->flowKey.dst_port = (icmp->icmp_type << 8) + icmp->icmp_code;
@@ -1115,7 +1065,7 @@ REDO_IPPROTO:
             }
 
             dbg_assert(dataptr <= eodata);
-            payloadSize = (ptrdiff_t)(eodata - dataptr);
+            payloadSize = (ptrdiff_t)(ipPayloadEnd - dataptr);
             if (payloadSize > 0) payload = (void *)dataptr;
 
             Node->flowKey.dst_port = (icmp6->icmp6_type << 8) + icmp6->icmp6_code;
@@ -1135,7 +1085,7 @@ REDO_IPPROTO:
             // move IP to tun IP
             Node->tun_src_addr = Node->flowKey.src_addr;
             Node->tun_dst_addr = Node->flowKey.dst_addr;
-            Node->tun_proto = IPPROTO_IPIP;
+            Node->tun_proto = IPPROTO_IPV6;
             Node->tun_ip_version = Node->flowKey.version;
 
             dbg_printf("  IPIPv6 tunnel - inner IPv6:\n");
@@ -1217,7 +1167,7 @@ REDO_IPPROTO:
                     optionSize += 4;
                 dataptr += optionSize;
 
-                payloadSize = (ptrdiff_t)(eodata - dataptr);
+                payloadSize = (ptrdiff_t)(ipPayloadEnd - dataptr);
                 if (payloadSize > 0) payload = (void *)dataptr;
 
                 ProcessOtherFlow(packetParam, Node, payload, payloadSize);
@@ -1248,7 +1198,7 @@ REDO_IPPROTO:
             // not handled transport protocol
             // raw flow
             dbg_assert(dataptr <= eodata);
-            payloadSize = (ptrdiff_t)(eodata - dataptr);
+            payloadSize = (ptrdiff_t)(ipPayloadEnd - dataptr);
             if (payloadSize > 0) payload = (void *)dataptr;
 
             dbg_printf("  raw proto: %u, payload size: %zu\n", IPproto, payloadSize);
