@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2024, Peter Haag
+ *  Copyright (c) 2009-2025, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -210,9 +210,9 @@ static void send_data(void *engine, timeWindow_t *timeWindow, uint64_t limitReco
     int reducer = 0;
 
     if (timeWindow) {
-        twin_msecFirst = timeWindow->first * 1000LL;
-        if (timeWindow->last)
-            twin_msecLast = timeWindow->last * 1000LL;
+        twin_msecFirst = timeWindow->msecFirst;
+        if (timeWindow->msecLast)
+            twin_msecLast = timeWindow->msecLast;
         else
             twin_msecLast = 0x7FFFFFFFFFFFFFFFLL;
     } else {
@@ -222,7 +222,7 @@ static void send_data(void *engine, timeWindow_t *timeWindow, uint64_t limitReco
     // Get the first file handle
     nffile = GetNextFile(NULL);
     if (!nffile) {
-        LogError("GetNextFile() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        LogError("GetNextFile() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return;
     }
     if (nffile == NULL) {
@@ -234,7 +234,7 @@ static void send_data(void *engine, timeWindow_t *timeWindow, uint64_t limitReco
     peer.send_buffer = malloc(UDP_PACKET_SIZE);
     peer.flush = 0;
     if (!peer.send_buffer) {
-        LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         CloseFile(nffile);
         DisposeFile(nffile);
         return;
@@ -254,7 +254,7 @@ static void send_data(void *engine, timeWindow_t *timeWindow, uint64_t limitReco
 
     recordHandle_t *recordHandle = calloc(1, sizeof(recordHandle_t));
     if (!recordHandle) {
-        LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return;
     }
 
@@ -286,7 +286,7 @@ static void send_data(void *engine, timeWindow_t *timeWindow, uint64_t limitReco
         uint32_t sumSize = 0;
         for (int i = 0; i < dataBlock->NumRecords; i++) {
             if ((sumSize + record_ptr->size) > dataBlock->size || (record_ptr->size < sizeof(record_header_t))) {
-                LogError("Corrupt data file. Inconsistent block size in %s line %d\n", __FILE__, __LINE__);
+                LogError("Corrupt data file. Inconsistent block size in %s line %d", __FILE__, __LINE__);
                 exit(255);
             }
             sumSize += record_ptr->size;
@@ -337,7 +337,7 @@ static void send_data(void *engine, timeWindow_t *timeWindow, uint64_t limitReco
                         int err = FlushBuffer(confirm);
 
                         if (err < 0) {
-                            perror("Error sending data");
+                            LogError("Error sending data");
                             CloseFile(nffile);
                             DisposeFile(nffile);
                             return;
@@ -437,9 +437,7 @@ static void send_data(void *engine, timeWindow_t *timeWindow, uint64_t limitReco
 }  // End of send_data
 
 int main(int argc, char **argv) {
-    struct stat stat_buff;
     char *ffile, *filter, *tstring;
-    int c, confirm, ffd, ret, netflow_version, distribution;
     unsigned int delay, sockbuff_size;
     timeWindow_t *timeWindow;
     flist_t flist;
@@ -457,11 +455,12 @@ int main(int argc, char **argv) {
 
     delay = 1;
     sockbuff_size = 0;
-    netflow_version = 9;
     verbose = 0;
-    confirm = 0;
-    distribution = 0;
+    int netflow_version = 9;
     uint64_t count = 0;
+    int confirm = 0;
+    int distribution = 0;
+    int c = 0;
     while ((c = getopt(argc, argv, "46EhH:i:K:L:p:S:d:c:b:j:r:f:t:v:z:VY")) != EOF) {
         switch (c) {
             case 'h':
@@ -522,6 +521,7 @@ int main(int argc, char **argv) {
                 sockbuff_size = atoi(optarg);
                 break;
             case 'f':
+                if (!CheckPath(optarg, S_IFREG)) exit(255);
                 ffile = optarg;
                 break;
             case 't':
@@ -566,27 +566,10 @@ int main(int argc, char **argv) {
     if (peer.hostname == NULL) peer.hostname = DEFAULTHOSTNAME;
 
     if (!filter && ffile) {
-        if (stat(ffile, &stat_buff)) {
-            perror("Can't stat file");
+        filter = ReadFilter(ffile);
+        if (filter == NULL) {
             exit(255);
         }
-        filter = (char *)malloc(stat_buff.st_size);
-        if (!filter) {
-            perror("Memory error");
-            exit(255);
-        }
-        ffd = open(ffile, O_RDONLY);
-        if (ffd < 0) {
-            perror("Can't open file");
-            exit(255);
-        }
-        ret = read(ffd, (void *)filter, stat_buff.st_size);
-        if (ret < 0) {
-            perror("Error reading file");
-            close(ffd);
-            exit(255);
-        }
-        close(ffd);
     }
 
     if (!filter) filter = "any";
