@@ -59,7 +59,7 @@ extern char influxdb_url[1024];
 static char influxdb_measurement[] = "nfsen_stats";
 #endif
 
-#if HAVE_RRDVERSION > 8
+#if RRD_NEEDSCONST
 #define rrdchar const char
 #else
 #define rrdchar char
@@ -251,18 +251,17 @@ static void SetupProfileChannels(char *profile_datadir, char *profile_statdir, p
     if ((profile_param->profiletype & 4) == 0) {  // no shadow profile
         int is_alert = (profile_param->profiletype & 8) == 8;
         if (!is_alert && subdir_index && strlen(filename) == 19 && (strncmp(filename, "nfcapd.", 7) == 0)) {
-            char *p = &filename[7];  // points to ISO timstamp in filename
+            char *p = &filename[7];  // points to ISO timestamp in filename
             time_t t = ISO2UNIX(p);
             struct tm *t_tm = localtime(&t);
-            char error[255];
 
             subdir = GetSubDir(t_tm);
             if (!subdir) {
                 // failed to generate subdir path - put flows into base directory
                 LogError("Failed to create subdir path!");
             }
-            if (!SetupSubDir(path, subdir, error, 255)) {
-                LogError("Failed to create subdir path: '%s'", error);
+            if (!SetupSubDir(path, subdir)) {
+                LogError("Failed to create subdir path");
                 // nothing else need to be done, as subdir == NULL means put files into channel directory
             }
         }
@@ -328,8 +327,8 @@ static void SetupProfileChannels(char *profile_datadir, char *profile_statdir, p
 
     memset((void *)&profile_channels[num_channels].stat_record, 0, sizeof(stat_record_t));
 
-    profile_channels[num_channels].stat_record.firstseen = 0x7fffffffffffffffLL;
-    profile_channels[num_channels].stat_record.lastseen = 0;
+    profile_channels[num_channels].stat_record.msecFirstSeen = 0x7fffffffffffffffLL;
+    profile_channels[num_channels].stat_record.msecLastSeen = 0;
 
     num_channels++;
 
@@ -371,7 +370,7 @@ void VerifyFiles(void) {
     for (unsigned num = 0; num < num_channels; num++) {
         if (profile_channels[num].wfile) {
             int err = QueryFile(profile_channels[num].wfile, 1);
-            LogError("Veryfied %s: err: %d\n", profile_channels[num].wfile, err);
+            LogError("Verified %s: err: %d\n", profile_channels[num].wfile, err);
         }
     }
 
