@@ -686,7 +686,7 @@ static void InitFormatParser(void) {
     token_list = (struct token_list_s *)calloc(1, max_token_index * sizeof(struct token_list_s));
     if (!token_list) {
         LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
-        exit(255);
+        exit(EXIT_FAILURE);
     }
 
 }  // End of InitFormatParser
@@ -718,7 +718,7 @@ static void AddToken(int index) {
         token_list = (struct token_list_s *)realloc(token_list, max_token_index * sizeof(struct token_list_s));
         if (!token_list) {
             LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
-            exit(255);
+            exit(EXIT_FAILURE);
         }
     }
 
@@ -1095,16 +1095,26 @@ static char *String_ja3(char *streamPtr, recordHandle_t *recordHandle) {
         return streamPtr;
     }
 
-    char *ja3 = recordHandle->extensionList[JA3index];
-    ssl_t *ssl = recordHandle->extensionList[SSLindex];
+    payloadHandle_t *payloadHandle = (payloadHandle_t *)recordHandle->extensionList[EXinPayloadHandle];
+    if (payloadHandle == NULL) {
+        payloadHandle = calloc(1, sizeof(payloadHandle_t));
+        if (!payloadHandle) {
+            LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
+            exit(EXIT_FAILURE);
+        } else {
+            recordHandle->extensionList[EXinPayloadHandle] = payloadHandle;
+        }
+    }
+    ssl_t *ssl = payloadHandle->ssl;
+    char *ja3 = payloadHandle->ja3;
     if (ja3 == NULL) {
         if (ssl == NULL) {
             uint32_t payloadLength = ExtensionLength(payload);
             ssl = sslProcess(payload, payloadLength);
-            recordHandle->extensionList[SSLindex] = ssl;
+            payloadHandle->ssl = ssl;
         }
         ja3 = ja3Process(ssl, NULL);
-        recordHandle->extensionList[JA3index] = ja3;
+        payloadHandle->ja3 = ja3;
         if (ssl == NULL || ja3 == NULL) {
             AddString("no-ja3");
             return streamPtr;
@@ -1127,16 +1137,27 @@ static char *String_ja4(char *streamPtr, recordHandle_t *recordHandle) {
         return streamPtr;
     }
 
-    ja4_t *ja4 = recordHandle->extensionList[JA4index];
-    ssl_t *ssl = recordHandle->extensionList[SSLindex];
+    payloadHandle_t *payloadHandle = (payloadHandle_t *)recordHandle->extensionList[EXinPayloadHandle];
+    if (payloadHandle == NULL) {
+        payloadHandle = calloc(1, sizeof(payloadHandle_t));
+        if (!payloadHandle) {
+            LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
+            exit(EXIT_FAILURE);
+        } else {
+            recordHandle->extensionList[EXinPayloadHandle] = payloadHandle;
+        }
+    }
+
+    ssl_t *ssl = payloadHandle->ssl;
+    ja4_t *ja4 = payloadHandle->ja4;
     if (ja4 == NULL) {
         if (ssl == NULL) {
             uint32_t payloadLength = ExtensionLength(payload);
             ssl = sslProcess(payload, payloadLength);
-            recordHandle->extensionList[SSLindex] = ssl;
+            payloadHandle->ssl = ssl;
         }
         ja4 = ja4Process(ssl, genericFlow->proto);
-        recordHandle->extensionList[JA4index] = ja4;
+        payloadHandle->ja4 = ja4;
         if (ssl == NULL || ja4 == NULL) {
             AddString("no-ja4");
             return streamPtr;
@@ -1162,11 +1183,22 @@ static char *String_tlsVersion(char *streamPtr, recordHandle_t *recordHandle) {
         return streamPtr;
     }
 
-    ssl_t *ssl = recordHandle->extensionList[SSLindex];
+    payloadHandle_t *payloadHandle = (payloadHandle_t *)recordHandle->extensionList[EXinPayloadHandle];
+    if (payloadHandle == NULL) {
+        payloadHandle = calloc(1, sizeof(payloadHandle_t));
+        if (!payloadHandle) {
+            LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
+            exit(EXIT_FAILURE);
+        } else {
+            recordHandle->extensionList[EXinPayloadHandle] = payloadHandle;
+        }
+    }
+
+    ssl_t *ssl = payloadHandle->ssl;
     if (ssl == NULL) {
         uint32_t payloadLength = ExtensionLength(payload);
         ssl = sslProcess(payload, payloadLength);
-        recordHandle->extensionList[SSLindex] = ssl;
+        payloadHandle->ssl = ssl;
         if (ssl == NULL) {
             AddChar('0');
             return streamPtr;
@@ -1214,11 +1246,22 @@ static char *String_sniName(char *streamPtr, recordHandle_t *recordHandle) {
         return streamPtr;
     }
 
-    ssl_t *ssl = recordHandle->extensionList[SSLindex];
+    payloadHandle_t *payloadHandle = (payloadHandle_t *)recordHandle->extensionList[EXinPayloadHandle];
+    if (payloadHandle == NULL) {
+        payloadHandle = calloc(1, sizeof(payloadHandle_t));
+        if (!payloadHandle) {
+            LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
+            exit(EXIT_FAILURE);
+        } else {
+            recordHandle->extensionList[EXinPayloadHandle] = payloadHandle;
+        }
+    }
+
+    ssl_t *ssl = payloadHandle->ssl;
     if (ssl == NULL) {
         uint32_t payloadLength = ExtensionLength(payload);
         ssl = sslProcess(payload, payloadLength);
-        recordHandle->extensionList[SSLindex] = ssl;
+        payloadHandle->ssl = ssl;
         if (ssl == NULL) {
             AddChar('0');
             return streamPtr;
