@@ -32,6 +32,7 @@
 
 #include <arpa/inet.h>
 #include <ctype.h>
+#include <errno.h>
 #include <inttypes.h>
 #include <netinet/in.h>
 #include <stddef.h>
@@ -43,6 +44,7 @@
 #include <time.h>
 
 #include "config.h"
+#include "dns/codec.h"
 #include "dns/dns.h"
 #include "exporter.h"
 #include "ifvrf.h"
@@ -54,6 +56,7 @@
 #include "nffile.h"
 #include "nfxV3.h"
 #include "output_util.h"
+#include "payload/dns/output_dns.h"
 #include "ssl/ssl.h"
 #include "tor/tor.h"
 #include "userio.h"
@@ -525,7 +528,7 @@ static void stringsEXsampler(FILE *stream, void *extensionRecord) {
 static void stringsEXobservation(FILE *stream, void *extensionRecord) {
     EXobservation_t *observation = (EXobservation_t *)extensionRecord;
     fprintf(stream,
-            "  obs domainID =          0x%05x\n"
+            "  obs domainID =            0x%05x\n"
             "  obs pointID  =       0x%010llx\n",
             observation->domainID, (long long unsigned)observation->pointID);
 
@@ -707,14 +710,25 @@ static void stringsEXnbarApp(FILE *stream, void *extensionRecord) {
 
 }  // End of stringsEXnbarApp
 
-static void inoutPayload(FILE *stream, recordHandle_t *recordHandle, uint8_t *payload, uint32_t length);
+static void inoutPayload(FILE *stream, recordHandle_t *recordHandle, payloadHandle_t *payloadHandle, uint8_t *payload, uint32_t length, char *prefix);
 
 static void stringsEXinPayload(FILE *stream, recordHandle_t *recordHandle, void *extensionRecord) {
     EXinPayload_t *inPayload = (EXinPayload_t *)recordHandle->extensionList[EXinPayloadID];
     uint32_t payloadLength = ExtensionLength(inPayload);
 
     fprintf(stream, "  in payload   =         %10u\n", payloadLength);
-    inoutPayload(stream, recordHandle, inPayload, payloadLength);
+
+    payloadHandle_t *payloadHandle = (payloadHandle_t *)recordHandle->extensionList[EXinPayloadHandle];
+    if (!payloadHandle) {
+        payloadHandle = calloc(1, sizeof(payloadHandle_t));
+        if (!payloadHandle) {
+            LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
+            exit(EXIT_FAILURE);
+        } else {
+            recordHandle->extensionList[EXinPayloadHandle] = payloadHandle;
+        }
+    }
+    inoutPayload(stream, recordHandle, payloadHandle, inPayload, payloadLength, "in");
 }  // End of stringsEXinPayload
 
 static void stringsEXoutPayload(FILE *stream, recordHandle_t *recordHandle, void *extensionRecord) {
@@ -722,33 +736,65 @@ static void stringsEXoutPayload(FILE *stream, recordHandle_t *recordHandle, void
     uint32_t payloadLength = ExtensionLength(outPayload);
 
     fprintf(stream, "  out payload  =         %10u\n", payloadLength);
-    inoutPayload(stream, recordHandle, outPayload, payloadLength);
+
+    payloadHandle_t *payloadHandle = (payloadHandle_t *)recordHandle->extensionList[EXoutPayloadHandle];
+    if (!payloadHandle) {
+        payloadHandle = calloc(1, sizeof(payloadHandle_t));
+        if (!payloadHandle) {
+            LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
+            exit(EXIT_FAILURE);
+        } else {
+            recordHandle->extensionList[EXinPayloadHandle] = payloadHandle;
+        }
+    }
+    inoutPayload(stream, recordHandle, payloadHandle, outPayload, payloadLength, "out");
 }  // end of stringsExoutPayload
 
-static void inoutPayload(FILE *stream, recordHandle_t *recordHandle, uint8_t *payload, uint32_t length) {
+static void inoutPayload(FILE *stream, recordHandle_t *recordHandle, payloadHandle_t *payloadHandle, uint8_t *payload, uint32_t payloadLength,
+                         char *prefix) {
     EXgenericFlow_t *genericFlow = (EXgenericFlow_t *)recordHandle->extensionList[EXgenericFlowID];
 
-    int max = length;
     if (genericFlow && (genericFlow->srcPort == 53 || genericFlow->dstPort == 53)) {
-        content_decode_dns(stream, genericFlow->proto, (uint8_t *)payload, length);
+        void *dnsDecoded = payloadHandle->dns;
+        if (dnsDecoded == NULL) {
+            if (genericFlow->proto == IPPROTO_TCP) {
+                dnsDecoded = dnsPayloadDecode(payload + 2, payloadLength - 2);
+            } else {
+                dnsDecoded = dnsPayloadDecode(payload, payloadLength);
+            }
+            payloadHandle->dns = dnsDecoded;
+            dns_print_result(stream, (dns_query_t *)dnsDecoded);
+        }
+        return;
     }
 
+    // ascii text can be printed as string incl. \n \r etc. nullBytes at the End of the string are allowed
+    // otherwise DumpHex() the payload.
     int ascii = 1;
-    for (int i = 0; i < max; i++) {
+    int nullBytes = 0;
+    for (int i = 0; i < payloadLength; i++) {
         if ((payload[i] < ' ' || payload[i] > '~') && payload[i] != '\n' && payload[i] != '\r' && payload[i] != 0x09) {
+            if (payload[i] == '\0') {
+                nullBytes++;
+            } else {
+                ascii = 0;
+                break;
+            }
+        } else if (nullBytes) {
             ascii = 0;
+            break;
         }
     }
 
     if (ascii) {
-        fprintf(stream, "%.*s\n", max, payload);
+        fprintf(stream, "%.*s\n", payloadLength, payload);
     } else if (genericFlow->proto == IPPROTO_TCP) {
-        ssl_t *ssl = recordHandle->extensionList[SSLindex];
+        ssl_t *ssl = payloadHandle->ssl;
         if (ssl == NULL) {
-            ssl = sslProcess(payload, length);
-            recordHandle->extensionList[SSLindex] = ssl;
+            ssl = sslProcess(payload, payloadLength);
+            payloadHandle->ssl = ssl;
             if (ssl == NULL) {
-                DumpHex(stream, payload, max);
+                DumpHex(stream, payload, payloadLength);
                 return;
             }
         }
@@ -756,50 +802,50 @@ static void inoutPayload(FILE *stream, recordHandle_t *recordHandle, uint8_t *pa
         // ssl is defined
         switch (ssl->tlsCharVersion[0]) {
             case 's':
-                fprintf(stream, "  TLS version  =              SSL %c  \n", ssl->tlsCharVersion[1]);
+                fprintf(stream, "    TLS vers   =              SSL %c  \n", ssl->tlsCharVersion[1]);
                 break;
             case '1':
-                fprintf(stream, "  TLS version  =            TLS 1.%c\n", ssl->tlsCharVersion[1]);
+                fprintf(stream, "    TLS vers   =            TLS 1.%c\n", ssl->tlsCharVersion[1]);
                 break;
             default:
-                fprintf(stream, "  TLS version  =              0x%4x\n", ssl->tlsVersion);
+                fprintf(stream, "    TLS vers   =              0x%4x\n", ssl->tlsVersion);
                 break;
         }
 
-        if (ssl->sniName[0]) fprintf(stream, "  sni name     =  %s\n", ssl->sniName);
+        if (ssl->sniName[0]) fprintf(stream, "    sni name   =  %s\n", ssl->sniName);
 
-        char *ja3 = recordHandle->extensionList[JA3index];
+        char *ja3 = payloadHandle->ja3;
         if (ja3 == NULL) {
             ja3 = ja3Process(ssl, NULL);
-            recordHandle->extensionList[JA3index] = ja3;
+            payloadHandle->ja3 = ja3;
         }
         if (ja3) {
             if (ssl->type == CLIENTssl) {
-                fprintf(stream, "  ja3 hash     =  %s\n", ja3);
+                fprintf(stream, "    ja3 hash   =  %s\n", ja3);
             } else {
-                fprintf(stream, "  ja3s hash    =  %s\n", ja3);
+                fprintf(stream, "    ja3s hash  =  %s\n", ja3);
             }
         }
 
-        ja4_t *ja4 = recordHandle->extensionList[JA4index];
+        ja4_t *ja4 = payloadHandle->ja4;
         if (ja4 == NULL) {
             if (ssl->type == CLIENTssl) {
                 ja4 = ja4Process(ssl, genericFlow->proto);
             } else {
                 ja4 = ja4sProcess(ssl, genericFlow->proto);
             }
-            recordHandle->extensionList[JA4index] = ja4;
+            payloadHandle->ja4 = ja4;
         }
 
         if (ja4) {
             if (ja4->type == TYPE_JA4)
-                fprintf(stream, "  ja4 hash     =  %s\n", ja4->string);
+                fprintf(stream, "    ja4 hash   =  %s\n", ja4->string);
             else
-                fprintf(stream, "  ja4s hash    =  %s\n", ja4->string);
+                fprintf(stream, "    ja4s hash  =  %s\n", ja4->string);
         }
     }
 
-    DumpHex(stream, payload, max);
+    DumpHex(stream, payload, payloadLength);
 }  // End of inoutPayload
 
 static void stringsEXpfinfo(FILE *stream, void *extensionRecord) {
@@ -911,6 +957,9 @@ void raw_record(FILE *stream, recordHandle_t *recordHandle, outputParams_t *outp
         }
     */
 
+    if (recordHandle->extensionList[EXasRoutingID] == NULL && outputParam->hasGeoDB)
+        recordHandle->extensionList[EXasRoutingID] = recordHandle->localStack;
+
     int processed = 0;
     int doInputPayload = 0;
     int doOutputPayload = 0;
