@@ -74,14 +74,15 @@ static void DumpRecord(recordHandle_t *recordHandle) {
     }
 
     printf("Count: %" PRIu64 "\n", recordHandle->flowCount);
-    ssl_t *ssl = recordHandle->extensionList[SSLindex];
+    payloadHandle_t *payloadHandle = recordHandle->extensionList[EXinPayloadHandle];
+    ssl_t *ssl = payloadHandle->ssl;
     if (ssl) {
         printf("SSL version : %c.%c\n", ssl->tlsCharVersion[0], ssl->tlsCharVersion[1]);
         printf("SSL SNI     : %s\n", ssl->sniName);
     }
-    char *s = recordHandle->extensionList[JA3index];
+    char *s = payloadHandle->ja3;
     printf("Ja3: %s\n", s != NULL ? s : "no ja3");
-    ja4_t *ja4 = recordHandle->extensionList[JA4index];
+    ja4_t *ja4 = payloadHandle->ja4;
     if (ja4) {
         switch (ja4->type) {
             case TYPE_JA4:
@@ -892,24 +893,25 @@ static void runTest(void) {
     CheckFilter("dst tun ip fe80::2110:abcd:1235:fffc", recordHandle, 0);
     CheckFilter("tun ip fe80::2110:abcd:1235:fffe", recordHandle, 1);
 
-    ssl_t ssl = {.type = CLIENTssl, .sniName = "example.com", .protocolVersion = 0x0303, .tlsCharVersion[0] = '1', .tlsCharVersion[1] = '2'};
     CheckFilter("payload ssl defined", recordHandle, 0);
-    recordHandle->extensionList[SSLindex] = (void *)&ssl;
+    ssl_t ssl = {.type = CLIENTssl, .sniName = "example.com", .protocolVersion = 0x0303, .tlsCharVersion[0] = '1', .tlsCharVersion[1] = '2'};
+    payloadHandle_t payloadHandle = {.ssl = &ssl};
+    recordHandle->extensionList[EXinPayloadHandle] = &payloadHandle;
     CheckFilter("payload ssl defined", recordHandle, 1);
     CheckFilter("payload tls version 1.2", recordHandle, 1);
     CheckFilter("payload tls version 1.3", recordHandle, 0);
     CheckFilter("payload ssl sni example", recordHandle, 1);
     CheckFilter("payload ssl sni nonexist", recordHandle, 0);
-    recordHandle->extensionList[SSLindex] = NULL;
+    payloadHandle.ssl = NULL;
     CheckFilter("payload ssl sni example", recordHandle, 0);
 
     // ja3
-    recordHandle->extensionList[JA3index] = "123456789abcdef0123456789abcdef0";
+    payloadHandle.ja3 = "123456789abcdef0123456789abcdef0";
     CheckFilter("payload ja3 123456789abcdef0123456789abcdef0", recordHandle, 1);
     CheckFilter("payload ja3 123456789abcdef0123456789abcdef1", recordHandle, 0);
     CheckFilter("payload ja3 023456789abcdef0123456789abcdef0", recordHandle, 0);
     CheckFilter("payload ja3 defined", recordHandle, 1);
-    recordHandle->extensionList[JA3index] = NULL;
+    payloadHandle.ja3 = NULL;
     CheckFilter("payload ja3 defined", recordHandle, 0);
 
     // ja4
@@ -920,41 +922,42 @@ static void runTest(void) {
     }
     ja4->type = TYPE_JA4;
     strcpy(ja4->string, "t13d1516h2_8daaf6152771_b186095e22b6");
-    recordHandle->extensionList[JA4index] = (void *)ja4;
+    payloadHandle.ja4 = ja4;
     CheckFilter("payload ja4 t13d1516h2_8daaf6152771_b186095e22b6", recordHandle, 1);
     CheckFilter("payload ja4 q13d1516h2_8daaf6152771_b186095e22b6", recordHandle, 0);
     CheckFilter("payload ja4 t13d1516h2_8daaf6152771_ccc6095e22b6", recordHandle, 0);
     CheckFilter("payload ja4 defined", recordHandle, 1);
-    recordHandle->extensionList[JA4index] = NULL;
+    payloadHandle.ja4 = NULL;
     CheckFilter("payload ja4 defined", recordHandle, 0);
 
 #ifdef BUILDJA4
     // ja4s
     ja4->type = TYPE_JA4S;
-    recordHandle->extensionList[JA4index] = (void *)ja4;
+    payloadHandle.ja4 = ja4;
     strcpy(ja4->string, "t120400_C030_4e8089608790");
     CheckFilter("payload ja4s t120400_C030_4e8089608790", recordHandle, 1);
     CheckFilter("payload ja4s q120400_C030_4e8089608790", recordHandle, 0);
     CheckFilter("payload ja4s t120400_C030_cccc89608790", recordHandle, 0);
     CheckFilter("payload ja4 defined", recordHandle, 1);
-    recordHandle->extensionList[JA4index] = NULL;
+    payloadHandle.ja4 = NULL;
     CheckFilter("payload ja4 defined", recordHandle, 0);
 #endif
+    recordHandle->extensionList[EXinPayloadHandle] = NULL;
 
     // local (processed) extension
     // geo location
     // src
-    recordHandle->geo[0] = 'C';
-    recordHandle->geo[1] = 'H';
+    recordHandle->geo[OFFgeoSrcIP - OFFgeo] = 'C';
+    recordHandle->geo[OFFgeoSrcIP - OFFgeo + 1] = 'H';
     // dst
-    recordHandle->geo[2] = 'D';
-    recordHandle->geo[3] = 'E';
+    recordHandle->geo[OFFgeoDstIP - OFFgeo] = 'D';
+    recordHandle->geo[OFFgeoDstIP - OFFgeo + 1] = 'E';
     // src nat
-    recordHandle->geo[4] = 'U';
-    recordHandle->geo[5] = 'S';
+    recordHandle->geo[OFFgeoSrcNatIP - OFFgeo] = 'U';
+    recordHandle->geo[OFFgeoSrcNatIP - OFFgeo + 1] = 'S';
     // dst nat
-    recordHandle->geo[6] = 'A';
-    recordHandle->geo[7] = 'T';
+    recordHandle->geo[OFFgeoDstNatIP - OFFgeo] = 'A';
+    recordHandle->geo[OFFgeoDstNatIP - OFFgeo + 1] = 'T';
 
     CheckFilter("src geo CH", recordHandle, 1);
     CheckFilter("src geo CD", recordHandle, 0);
